#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Anthony Bond <ajbond2005@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_firewall_info
short_description: Gather information about DigitalOcean firewalls
description:
    - This module can be used to gather information about DigitalOcean firewalls.
    - This module was called C(digital_ocean_firewall_facts) before Ansible 2.9. The usage did not change.
author: "Anthony Bond (@BondAnthony)"
options:
  name:
    description:
     - Firewall rule name that can be used to identify and reference a specific firewall rule.
    required: false
    type: str
requirements:
  - "python >= 2.6"
extends_documentation_fragment:
- community.digitalocean.digital_ocean.documentation

"""


EXAMPLES = r"""
- name: Gather information about all firewalls
  community.digitalocean.digital_ocean_firewall_info:
    oauth_token: "{{ oauth_token }}"

- name: Gather information about a specific firewall by name
  community.digitalocean.digital_ocean_firewall_info:
    oauth_token: "{{ oauth_token }}"
    name: "firewall_name"

- name: Gather information from a firewall rule
  community.digitalocean.digital_ocean_firewall_info:
    name: SSH
  register: resp_out

- set_fact:
    firewall_id: "{{ resp_out.data.id }}"

- debug:
    msg: "{{ firewall_id }}"
"""


RETURN = r"""
data:
    description: DigitalOcean firewall information
    returned: success
    type: list
    elements: dict
    sample: [
        {
            "id": "435tbg678-1db53-32b6-t543-28322569t252",
            "name": "metrics",
            "status": "succeeded",
            "inbound_rules": [
                {
                    "protocol": "tcp",
                    "ports": "9100",
                    "sources": {
                        "addresses": [
                            "1.1.1.1"
                        ]
                    }
                }
            ],
            "outbound_rules": [],
            "created_at": "2018-01-15T07:04:25Z",
            "droplet_ids": [
                87426985
            ],
            "tags": [],
            "pending_changes": []
        },
    ]
"""

from traceback import format_exc
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)
from ansible.module_utils._text import to_native


def core(module):
    firewall_name = module.params.get("name", None)
    rest = DigitalOceanHelper(module)
    base_url = "firewalls?"

    response = rest.get("%s" % base_url)
    status_code = response.status_code
    if status_code != 200:
        module.fail_json(msg="Failed to retrieve firewalls from Digital Ocean")
    firewalls = rest.get_paginated_data(base_url=base_url, data_key_name="firewalls")

    if firewall_name is not None:
        rule = {}
        for firewall in firewalls:
            if firewall["name"] == firewall_name:
                rule.update(firewall)
        firewalls = [rule]
        module.exit_json(changed=False, data=firewalls)
    else:
        module.exit_json(changed=False, data=firewalls)


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        name=dict(type="str", required=False),
    )
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
    if module._name in (
        "digital_ocean_firewall_facts",
        "community.digitalocean.digital_ocean_firewall_facts",
    ):
        module.deprecate(
            "The 'digital_ocean_firewall_facts' module has been renamed to 'digital_ocean_firewall_info'",
            version="2.0.0",
            collection_name="community.digitalocean",
        )  # was Ansible 2.13

    try:
        core(module)
    except Exception as e:
        module.fail_json(msg=to_native(e), exception=format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualmachinescalesetinstance_info
version_added: "0.1.2"
short_description: Get Azure Virtual Machine Scale Set Instance facts
description:
    - Get facts of Azure Virtual Machine Scale Set VMs.

options:
    resource_group:
        description:
            - The name of the resource group.
        required: True
        type: str
    vmss_name:
        description:
            - The name of the VM scale set.
        required: True
        type: str
    instance_id:
        description:
            - The instance ID of the virtual machine.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: List VM instances in Virtual Machine ScaleSet
  azure_rm_virtualmachinescalesetinstance_info:
    resource_group: myResourceGroup
    vmss_name: myVMSS
'''

RETURN = '''
instances:
    description:
        - A list of dictionaries containing facts for Virtual Machine Scale Set VM.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/virtualMachineScaleSets/my
                     VMSS/virtualMachines/2"
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'tag1': 'abc' }
        instance_id:
            description:
                - Virtual Machine instance ID.
            returned: always
            type: str
            sample: 0
        name:
            description:
                - Virtual Machine name.
            returned: always
            type: str
            sample: myVMSS_2
        latest_model:
            description:
                - Whether applied latest model.
            returned: always
            type: bool
            sample: True
        provisioning_state:
            description:
                - Provisioning state of the Virtual Machine.
            returned: always
            type: str
            sample: Succeeded
        power_state:
            description:
                - Provisioning state of the Virtual Machine's power.
            returned: always
            type: str
            sample: running
        vm_id:
            description:
                - Virtual Machine ID
            returned: always
            type: str
            sample: 94a141a9-4530-46ac-b151-2c7ff09aa823
        image_reference:
            description:
                - Image reference
            returned: always
            type: dict
            sample: { "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myGroup/providers/Microsoft.Compute/galleries/
                        myGallery/images/myImage/versions/10.1.3"}
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.compute import ComputeManagementClient
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMVirtualMachineScaleSetVMInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            vmss_name=dict(
                type='str',
                required=True
            ),
            instance_id=dict(
                type='str'
            ),
            tags=dict(
                type='list',
                elements='str'
            )
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.mgmt_client = None
        self.resource_group = None
        self.vmss_name = None
        self.instance_id = None
        self.tags = None
        super(AzureRMVirtualMachineScaleSetVMInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):
        is_old_facts = self.module._name == 'azure_rm_virtualmachinescalesetinstance_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_virtualmachinescalesetinstance_facts' module has been renamed to" +
                                  " 'azure_rm_virtualmachinescalesetinstance_info'",
                                  version=(2.9, ))

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])
        self.mgmt_client = self.get_mgmt_svc_client(ComputeManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    is_track2=True,
                                                    api_version='2021-04-01')

        if (self.instance_id is None):
            self.results['instances'] = self.list()
        else:
            self.results['instances'] = self.get()
        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.mgmt_client.virtual_machine_scale_set_vms.get(resource_group_name=self.resource_group,
                                                                          vm_scale_set_name=self.vmss_name,
                                                                          instance_id=self.instance_id)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Virtual Machine Scale Set VM.')

        if response and self.has_tags(response.tags, self.tags):
            results.append(self.format_response(response))

        return results

    def list(self):
        items = None
        try:
            items = self.mgmt_client.virtual_machine_scale_set_vms.list(resource_group_name=self.resource_group,
                                                                        virtual_machine_scale_set_name=self.vmss_name)
            self.log("Response : {0}".format(items))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Virtual Machine ScaleSet VM.')

        results = []
        for item in items:
            if self.has_tags(item.tags, self.tags):
                results.append(self.format_response(item))
        return results

    def format_response(self, item):
        d = item.as_dict()

        iv = self.mgmt_client.virtual_machine_scale_set_vms.get_instance_view(resource_group_name=self.resource_group,
                                                                              vm_scale_set_name=self.vmss_name,
                                                                              instance_id=d.get('instance_id', None)).as_dict()
        power_state = ""
        for index in range(len(iv['statuses'])):
            code = iv['statuses'][index]['code'].split('/')
            if code[0] == 'PowerState':
                power_state = code[1]
                break
        d = {
            'resource_group': self.resource_group,
            'id': d.get('id', None),
            'tags': d.get('tags', None),
            'instance_id': d.get('instance_id', None),
            'latest_model': d.get('latest_model_applied', None),
            'name': d.get('name', None),
            'provisioning_state': d.get('provisioning_state', None),
            'power_state': power_state,
            'vm_id': d.get('vm_id', None),
            'image_reference': d.get('storage_profile').get('image_reference', None),
            'computer_name': d.get('os_profile').get('computer_name', None)
        }
        return d


def main():
    AzureRMVirtualMachineScaleSetVMInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: configuration
short_description: Resource module for Configuration
description:
- Manage operation update of the resource Configuration.
- API to enable/disable LSD settings.
version_added: '2.8.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  enableEPO:
    description: To enable/disable LSD ownership settings.
    type: bool
  enableRCM:
    description: To enable/disable random mac(RCM) settings. Please note that this flag
      will be set to false if enableEPO flag is false.
    type: bool
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    configuration.Configuration.update_configuration,

  - Paths used are
    put /api/v1/lsd/updateLsdSettings,

"""

EXAMPLES = r"""
- name: Update all
  cisco.ise.configuration:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    enableEPO: true
    enableRCM: true

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enableEPO": true,
      "enableRCM": true
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "enableEPO": true,
      "enableRCM": true
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the nxos_bgp_neighbor_address_family module
"""


class Bgp_neighbor_address_familyArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_bgp_neighbor_address_family module"""

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {
            "type": "dict",
            "options": {
                "as_number": {"type": "str"},
                "neighbors": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "neighbor_address": {"type": "str", "required": True},
                        "address_family": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "afi": {
                                    "type": "str",
                                    "choices": [
                                        "ipv4",
                                        "ipv6",
                                        "link-state",
                                        "vpnv4",
                                        "vpnv6",
                                        "l2vpn",
                                    ],
                                    "required": True,
                                },
                                "safi": {
                                    "type": "str",
                                    "choices": [
                                        "unicast",
                                        "multicast",
                                        "mvpn",
                                        "evpn",
                                    ],
                                },
                                "advertise_map": {
                                    "type": "dict",
                                    "options": {
                                        "route_map": {
                                            "type": "str",
                                            "required": True,
                                        },
                                        "exist_map": {"type": "str"},
                                        "non_exist_map": {"type": "str"},
                                    },
                                },
                                "advertisement_interval": {"type": "int"},
                                "allowas_in": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "max_occurences": {"type": "int"},
                                    },
                                },
                                "as_override": {"type": "bool"},
                                "capability": {
                                    "type": "dict",
                                    "options": {
                                        "additional_paths": {
                                            "type": "dict",
                                            "options": {
                                                "receive": {
                                                    "type": "str",
                                                    "choices": [
                                                        "enable",
                                                        "disable",
                                                    ],
                                                },
                                                "send": {
                                                    "type": "str",
                                                    "choices": [
                                                        "enable",
                                                        "disable",
                                                    ],
                                                },
                                            },
                                        },
                                    },
                                },
                                "default_originate": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "route_map": {"type": "str"},
                                    },
                                },
                                "disable_peer_as_check": {"type": "bool"},
                                "filter_list": {
                                    "type": "dict",
                                    "options": {
                                        "inbound": {"type": "str"},
                                        "outbound": {"type": "str"},
                                    },
                                },
                                "inherit": {
                                    "type": "dict",
                                    "options": {
                                        "template": {"type": "str"},
                                        "sequence": {"type": "int"},
                                    },
                                },
                                "maximum_prefix": {
                                    "type": "dict",
                                    "options": {
                                        "max_prefix_limit": {"type": "int"},
                                        "generate_warning_threshold": {"type": "int"},
                                        "restart_interval": {"type": "int"},
                                        "warning_only": {"type": "bool"},
                                    },
                                },
                                "next_hop_self": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "all_routes": {"type": "bool"},
                                    },
                                },
                                "next_hop_third_party": {"type": "bool"},
                                "prefix_list": {
                                    "type": "dict",
                                    "options": {
                                        "inbound": {"type": "str"},
                                        "outbound": {"type": "str"},
                                    },
                                },
                                "rewrite_evpn_rt_asn": {"type": "bool"},
                                "rewrite_rt_asn": {"type": "bool"},
                                "route_map": {
                                    "type": "dict",
                                    "options": {
                                        "inbound": {"type": "str"},
                                        "outbound": {"type": "str"},
                                    },
                                },
                                "route_reflector_client": {"type": "bool"},
                                "send_community": {
                                    "type": "dict",
                                    "mutually_exclusive": [
                                        ["both", "set"],
                                        ["extended", "both"],
                                        ["standard", "both"],
                                        ["standard", "set"],
                                    ],
                                    "options": {
                                        "set": {"type": "bool"},
                                        "extended": {"type": "bool"},
                                        "standard": {"type": "bool"},
                                        "both": {"type": "bool"},
                                    },
                                },
                                "soft_reconfiguration_inbound": {
                                    "type": "dict",
                                    "options": {
                                        "set": {"type": "bool"},
                                        "always": {"type": "bool"},
                                    },
                                },
                                "soo": {"type": "str"},
                                "suppress_inactive": {"type": "bool"},
                                "unsuppress_map": {"type": "str"},
                                "weight": {"type": "int"},
                            },
                        },
                    },
                },
                "vrfs": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "vrf": {"type": "str"},
                        "neighbors": {
                            "type": "list",
                            "elements": "dict",
                            "options": {
                                "neighbor_address": {
                                    "type": "str",
                                    "required": True,
                                },
                                "address_family": {
                                    "type": "list",
                                    "elements": "dict",
                                    "options": {
                                        "afi": {
                                            "type": "str",
                                            "choices": [
                                                "ipv4",
                                                "ipv6",
                                                "link-state",
                                                "vpnv4",
                                                "vpnv6",
                                                "l2vpn",
                                            ],
                                            "required": True,
                                        },
                                        "safi": {
                                            "type": "str",
                                            "choices": [
                                                "unicast",
                                                "multicast",
                                                "mvpn",
                                                "evpn",
                                            ],
                                        },
                                        "advertise_map": {
                                            "type": "dict",
                                            "options": {
                                                "route_map": {
                                                    "type": "str",
                                                    "required": True,
                                                },
                                                "exist_map": {"type": "str"},
                                                "non_exist_map": {"type": "str"},
                                            },
                                        },
                                        "advertisement_interval": {"type": "int"},
                                        "allowas_in": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "max_occurences": {"type": "int"},
                                            },
                                        },
                                        "as_override": {"type": "bool"},
                                        "capability": {
                                            "type": "dict",
                                            "options": {
                                                "additional_paths": {
                                                    "type": "dict",
                                                    "options": {
                                                        "receive": {
                                                            "type": "str",
                                                            "choices": [
                                                                "enable",
                                                                "disable",
                                                            ],
                                                        },
                                                        "send": {
                                                            "type": "str",
                                                            "choices": [
                                                                "enable",
                                                                "disable",
                                                            ],
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "default_originate": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "route_map": {"type": "str"},
                                            },
                                        },
                                        "disable_peer_as_check": {"type": "bool"},
                                        "filter_list": {
                                            "type": "dict",
                                            "options": {
                                                "inbound": {"type": "str"},
                                                "outbound": {"type": "str"},
                                            },
                                        },
                                        "inherit": {
                                            "type": "dict",
                                            "options": {
                                                "template": {"type": "str"},
                                                "sequence": {"type": "int"},
                                            },
                                        },
                                        "maximum_prefix": {
                                            "type": "dict",
                                            "options": {
                                                "max_prefix_limit": {"type": "int"},
                                                "generate_warning_threshold": {"type": "int"},
                                                "restart_interval": {"type": "int"},
                                                "warning_only": {"type": "bool"},
                                            },
                                        },
                                        "next_hop_self": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "all_routes": {"type": "bool"},
                                            },
                                        },
                                        "next_hop_third_party": {"type": "bool"},
                                        "prefix_list": {
                                            "type": "dict",
                                            "options": {
                                                "inbound": {"type": "str"},
                                                "outbound": {"type": "str"},
                                            },
                                        },
                                        "rewrite_evpn_rt_asn": {"type": "bool"},
                                        "rewrite_rt_asn": {"type": "bool"},
                                        "route_map": {
                                            "type": "dict",
                                            "options": {
                                                "inbound": {"type": "str"},
                                                "outbound": {"type": "str"},
                                            },
                                        },
                                        "route_reflector_client": {"type": "bool"},
                                        "send_community": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "extended": {"type": "bool"},
                                                "standard": {"type": "bool"},
                                                "both": {"type": "bool"},
                                            },
                                        },
                                        "soft_reconfiguration_inbound": {
                                            "type": "dict",
                                            "options": {
                                                "set": {"type": "bool"},
                                                "always": {"type": "bool"},
                                            },
                                        },
                                        "soo": {"type": "str"},
                                        "suppress_inactive": {"type": "bool"},
                                        "unsuppress_map": {"type": "str"},
                                        "weight": {"type": "int"},
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "parsed",
                "gathered",
                "rendered",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
#
# Copyright (c) 2020 Cole Neubauer, (@coleneubauer)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_aduser

version_added: "1.5.0"

short_description: Modify an Azure Active Directory user

description:
    - Create, delete, and update an Azure Active Directory user.

options:
    tenant:
        description:
            - The tenant ID.
        type: str
        required: True
    state:
        description:
            - State of the ad user. Use C(present) to create or update an ad user and C(absent) to delete an ad user.
        type: str
        default: present
        choices:
            - absent
            - present
    object_id:
        description:
            - The object id for the user.
            - Updates or deletes the user who has this object ID.
            - Mutually exclusive with I(user_principal_name), I(attribute_name), and I(odata_filter).
        type: str
    account_enabled:
        description:
            - A boolean determing whether or not the user account is enabled.
            - Used when either creating or updating a user account.
        type: bool
    display_name:
        description:
            - The display name of the user.
            - Used when either creating or updating a user account.
        type: str
    given_name:
        description:
            - The given name for the user.
            - Used when either creating or updating a user account.
        type: str
    surname:
        description:
            - The surname for the user.
            - Used when either creating or updating a user account.
        type: str
    immutable_id:
        description:
            - The immutable_id of the user.
            - Used when either creating or updating a user account.
        type: str
    mail:
        description:
            - The primary email address of the user.
            - Used when either creating or updating a user account.
        type: str
    mail_nickname:
        description:
            - The mail alias for the user.
            - Used when either creating or updating a user account.
        type: str
    password_profile:
        description:
            - The password for the user.
            - Used when either creating or updating a user account.
        type: str
    usage_location:
        description:
            - A two letter country code, ISO standard 3166.
            - Required for a user that will be assigned licenses due to legal requirement to check for availability of services in countries.
            - Used when either creating or updating a user account.
        type: str
    user_type:
        description:
            - A string value that can be used to classify user types in your directory, such as Member and Guest.
            - Used when either creating or updating a user account.
        type: str
    user_principal_name:
        description:
            - The principal name of the user.
            - Creates, updates, or deletes the user who has this principal name.
            - Mutually exclusive with I(object_id), I(attribute_name), and I(odata_filter).
        type: str
    attribute_name:
        description:
            - The name of an attribute that you want to match to I(attribute_value).
            - If I(attribute_name) is not a collection type it will update or delete the user where I(attribute_name) is equal to I(attribute_value).
            - If I(attribute_name) is a collection type it will update or delete the user where I(attribute_value) is in I(attribute_name).
            - Mutually exclusive with I(object_id), I(user_principal_name), and I(odata_filter).
            - Required together with I(attribute_value).
        type: str
    attribute_value:
        description:
            - The value to match I(attribute_name) to.
            - If I(attribute_name) is not a collection type it will update or delete the user where I(attribute_name) is equal to I(attribute_value).
            - If I(attribute_name) is a collection type it will update or delete the user where I(attribute_value) is in I(attribute_name).
            - Required together with I(attribute_name).
        type: str
    odata_filter:
        description:
            - Filter that can be used to specify a user to update or delete.
            - Mutually exclusive with I(object_id), I(attribute_name), and I(user_principal_name).
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Cole Neubauer(@coleneubauer)

'''

EXAMPLES = '''
- name: Create user
  azure_rm_aduser:
    user_principal_name: "{{ user_id }}"
    tenant: "{{ tenant_id }}"
    state: "present"
    account_enabled: "True"
    display_name: "Test_{{ user_principal_name }}_Display_Name"
    password_profile: "password"
    mail_nickname: "Test_{{ user_principal_name }}_mail_nickname"
    immutable_id: "{{ object_id }}"
    given_name: "First"
    surname: "Last"
    user_type: "Member"
    usage_location: "US"
    mail: "{{ user_principal_name }}@contoso.com"

- name: Update user with new value for account_enabled
  azure_rm_aduser:
    user_principal_name: "{{ user_id }}"
    tenant: "{{ tenant_id }}"
    state: "present"
    account_enabled: "False"

- name: Delete user
  azure_rm_aduser:
    user_principal_name: "{{ user_id }}"
    tenant: "{{ tenant_id }}"
    state: "absent"
'''

RETURN = '''
object_id:
    description:
        - The object_id for the user.
    type: str
    returned: always
    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
display_name:
    description:
        - The display name of the user.
    returned: always
    type: str
    sample: John Smith
user_principal_name:
    description:
        - The principal name of the user.
    returned: always
    type: str
    sample: jsmith@contoso.com
mail_nickname:
    description:
        - The mail alias for the user.
    returned: always
    type: str
    sample: jsmith
mail:
    description:
        - The primary email address of the user.
    returned: always
    type: str
    sample: John.Smith@contoso.com
account_enabled:
    description:
        - Whether the account is enabled.
    returned: always
    type: bool
    sample: False
user_type:
    description:
        - A string value that can be used to classify user types in your directory.
    returned: always
    type: str
    sample: Member
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase

try:
    from azure.graphrbac.models import UserUpdateParameters
    from azure.graphrbac.models import UserCreateParameters
    from azure.graphrbac.models import PasswordProfile
    from azure.graphrbac.models import GraphErrorException
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADUser(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            user_principal_name=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            object_id=dict(type='str'),
            attribute_name=dict(type='str'),
            attribute_value=dict(type='str'),
            odata_filter=dict(type='str'),
            account_enabled=dict(type='bool'),
            display_name=dict(type='str'),
            password_profile=dict(type='str', no_log=True),
            mail_nickname=dict(type='str'),
            immutable_id=dict(type='str'),
            usage_location=dict(type='str'),
            given_name=dict(type='str'),
            surname=dict(type='str'),
            user_type=dict(type='str'),
            mail=dict(type='str'),
            tenant=dict(type='str', required=True),
        )

        self.tenant = None
        self.user_principal_name = None
        self.state = None
        self.object_id = None
        self.attribute_name = None
        self.attribute_value = None
        self.odata_filter = None
        self.account_enabled = None
        self.display_name = None
        self.password_profile = None
        self.mail_nickname = None
        self.immutable_id = None
        self.usage_location = None
        self.given_name = None
        self.surname = None
        self.user_type = None
        self.mail = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(changed=False)

        mutually_exclusive = [['odata_filter', 'attribute_name', 'object_id', 'user_principal_name']]
        required_together = [['attribute_name', 'attribute_value']]
        required_one_of = [['odata_filter', 'attribute_name', 'object_id', 'user_principal_name']]

        super(AzureRMADUser, self).__init__(derived_arg_spec=self.module_arg_spec,
                                            supports_check_mode=False,
                                            supports_tags=False,
                                            mutually_exclusive=mutually_exclusive,
                                            required_together=required_together,
                                            required_one_of=required_one_of,
                                            is_ad_resource=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        try:
            client = self.get_graphrbac_client(self.tenant)

            ad_user = self.get_exisiting_user(client)

            if self.state == 'present':

                if ad_user:  # Update, changed

                    password = None

                    if self.password_profile:
                        password = PasswordProfile(password=self.password_profile)

                    should_update = False

                    if self.immutable_id and ad_user.immutable_id != self.immutable_id:
                        should_update = True
                    if should_update or self.usage_location and ad_user.usage_location != self.usage_location:
                        should_update = True
                    if should_update or self.given_name and ad_user.given_name != self.given_name:
                        should_update = True
                    if should_update or self.surname and ad_user.surname != self.surname:
                        should_update = True
                    if should_update or self.user_type and ad_user.user_type != self.user_type:
                        should_update = True
                    if should_update or self.account_enabled is not None and ad_user.account_enabled != self.account_enabled:
                        should_update = True
                    if should_update or self.display_name and ad_user.display_name != self.display_name:
                        should_update = True
                    if should_update or password:
                        should_update = True
                    if should_update or self.user_principal_name and ad_user.user_principal_name != self.user_principal_name:
                        should_update = True
                    if should_update or self.mail_nickname and ad_user.mail_nickname != self.mail_nickname:
                        should_update = True

                    if should_update:
                        parameters = UserUpdateParameters(immutable_id=self.immutable_id,
                                                          usage_location=self.usage_location,
                                                          given_name=self.given_name,
                                                          surname=self.surname,
                                                          user_type=self.user_type,
                                                          account_enabled=self.account_enabled,
                                                          display_name=self.display_name,
                                                          password_profile=password,
                                                          user_principal_name=self.user_principal_name,
                                                          mail_nickname=self.mail_nickname)

                        client.users.update(upn_or_object_id=ad_user.object_id, parameters=parameters)

                        self.results['changed'] = True

                        # Get the updated versions of the users to return
                        # the update method, has no return value so it needs to be explicitely returned in a call
                        ad_user = self.get_exisiting_user(client)

                    else:
                        self.results['changed'] = False

                else:  # Create, changed
                    password = PasswordProfile(password=self.password_profile)
                    parameters = UserCreateParameters(account_enabled=self.account_enabled,
                                                      display_name=self.display_name,
                                                      password_profile=password,
                                                      user_principal_name=self.user_principal_name,
                                                      mail_nickname=self.mail_nickname,
                                                      immutable_id=self.immutable_id,
                                                      usage_location=self.usage_location,
                                                      given_name=self.given_name,
                                                      surname=self.surname,
                                                      user_type=self.user_type,
                                                      mail=self.mail)
                    ad_user = client.users.create(parameters=parameters)
                    self.results['changed'] = True

                self.results['ad_user'] = self.to_dict(ad_user)

            elif self.state == 'absent':
                if ad_user:  # Delete, changed
                    client.users.delete(ad_user.object_id)
                    self.results['changed'] = True
                else:  # Do nothing unchanged
                    self.results['changed'] = False

        except GraphErrorException as e:
            self.fail("failed to get ad user info {0}".format(str(e)))

        return self.results

    def get_exisiting_user(self, client):
        ad_user = None

        try:
            if self.user_principal_name is not None:
                ad_user = client.users.get(self.user_principal_name)
            elif self.object_id is not None:
                ad_user = client.users.get(self.object_id)
            elif self.attribute_name is not None and self.attribute_value is not None:
                try:
                    ad_user = list(client.users.list(filter="{0} eq '{1}'".format(self.attribute_name, self.attribute_value)))[0]
                except GraphErrorException as e:
                    # the type doesn't get more specific. Could check the error message but no guarantees that message doesn't change in the future
                    # more stable to try again assuming the first error came from the attribute being a list
                    try:
                        ad_user = list(client.users.list(filter="{0}/any(c:c eq '{1}')".format(self.attribute_name, self.attribute_value)))[0]
                    except GraphErrorException as sub_e:
                        raise
            elif self.odata_filter is not None:  # run a filter based on user input to return based on any given attribute/query
                ad_user = list(client.users.list(filter=self.odata_filter))[0]
        except GraphErrorException as e:
            # User was not found
            err_msg = str(e)
            if err_msg == "Resource '{0}' does not exist or one of its queried reference-property objects are not present.".format(self.user_principal_name):
                ad_user = None
            else:
                raise
        return ad_user

    def to_dict(self, object):
        return dict(
            object_id=object.object_id,
            display_name=object.display_name,
            user_principal_name=object.user_principal_name,
            mail_nickname=object.mail_nickname,
            mail=object.mail,
            account_enabled=object.account_enabled,
            user_type=object.user_type
        )


def main():
    AzureRMADUser()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
# (c) 2018, Will Thames <will@thames.id.au>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import os

from ansible.errors import AnsibleAction
from ansible.errors import AnsibleActionFail
from ansible.errors import AnsibleError
from ansible.errors import AnsibleFileNotFound
from ansible.module_utils._text import to_text
from ansible.plugins.action import ActionBase
from ansible.utils.vars import merge_hash


class ActionModule(ActionBase):
    TRANSFERS_FILES = True

    def run(self, tmp=None, task_vars=None):
        """handler for s3_object operations

        This adds the magic that means 'src' can point to both a 'remote' file
        on the 'host' or in the 'files/' lookup path on the controller.
        """
        self._supports_async = True

        if task_vars is None:
            task_vars = dict()

        result = super().run(tmp, task_vars)
        del tmp  # tmp no longer has any effect

        source = self._task.args.get("src", None)

        try:
            new_module_args = self._task.args.copy()
            if source:
                source = os.path.expanduser(source)

                # For backward compatibility check if the file exists on the remote; it should take precedence
                if not self._remote_file_exists(source):
                    try:
                        source = self._loader.get_real_file(self._find_needle("files", source), decrypt=False)
                        new_module_args["src"] = source
                    except AnsibleFileNotFound:
                        # module handles error message for nonexistent files
                        new_module_args["src"] = source
                    except AnsibleError as e:
                        raise AnsibleActionFail(to_text(e))

            wrap_async = self._task.async_val and not self._connection.has_native_async
            # execute the s3_object module with the updated args
            result = merge_hash(
                result, self._execute_module(module_args=new_module_args, task_vars=task_vars, wrap_async=wrap_async)
            )

            if not wrap_async:
                # remove a temporary path we created
                self._remove_tmp_path(self._connection._shell.tmpdir)

        except AnsibleAction as e:
            result.update(e.result)
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_switch_access_control_lists
short_description: Resource module for networks _switch _accesscontrollists
description:
- Manage operation update of the resource networks _switch _accesscontrollists.
- Update the access control lists for a MS network.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  rules:
    description: An ordered array of the access control list rules (not including the
      default rule). An empty array will clear the rules.
    elements: dict
    suboptions:
      comment:
        description: Description of the rule (optional).
        type: str
      dstCidr:
        description: Destination IP address (in IP or CIDR notation) or 'any'.
        type: str
      dstPort:
        description: Destination port. Must be in the range of 1-65535 or 'any'. Default
          is 'any'.
        type: str
      ipVersion:
        description: IP address version (must be 'any', 'ipv4' or 'ipv6'). Applicable
          only if network supports IPv6. Default value is 'ipv4'.
        type: str
      policy:
        description: '''allow'' or ''deny'' traffic specified by this rule.'
        type: str
      protocol:
        description: The type of protocol (must be 'tcp', 'udp', or 'any').
        type: str
      srcCidr:
        description: Source IP address (in IP or CIDR notation) or 'any'.
        type: str
      srcPort:
        description: Source port. Must be in the range of 1-65535 or 'any'. Default
          is 'any'.
        type: str
      vlan:
        description: Incoming traffic VLAN. Must be in the range of 1-4095 or 'any'.
          Default is 'any'.
        type: str
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for switch updateNetworkSwitchAccessControlLists
  description: Complete reference of the updateNetworkSwitchAccessControlLists API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-access-control-lists
notes:
  - SDK Method used are
    switch.Switch.update_network_switch_access_control_lists,

  - Paths used are
    put /networks/{networkId}/switch/accessControlLists,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_switch_access_control_lists:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    networkId: string
    rules:
    - comment: Deny SSH
      dstCidr: 172.16.30/24
      dstPort: '22'
      ipVersion: ipv4
      policy: deny
      protocol: tcp
      srcCidr: 10.1.10.0/24
      srcPort: any
      vlan: '10'

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    [
      {
        "comment": "string",
        "policy": "string",
        "ipVersion": "string",
        "protocol": "string",
        "srcCidr": "string",
        "srcPort": "string",
        "dstCidr": "string",
        "dstPort": "string",
        "vlan": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: dms_replication_subnet_group
version_added: 1.0.0
short_description: creates or destroys a data migration services subnet group
description:
    - Creates or destroys a data migration services subnet group.
options:
    state:
        description:
            - State of the subnet group.
        default: present
        choices: ['present', 'absent']
        type: str
    identifier:
        description:
            - The name for the replication subnet group.
              This value is stored as a lowercase string.
              Must contain no more than 255 alphanumeric characters,
              periods, spaces, underscores, or hyphens. Must not be "default".
        type: str
        required: true
    description:
        description:
            - The description for the subnet group.
        type: str
        required: true
    subnet_ids:
        description:
            - A list containing the subnet ids for the replication subnet group,
              needs to be at least 2 items in the list.
        type: list
        elements: str
        required: true
author:
    - "Rui Moreira (@ruimoreira)"
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
"""

EXAMPLES = r"""
- community.aws.dms_replication_subnet_group:
    state: present
    identifier: "dev-sngroup"
    description: "Development Subnet Group asdasdas"
    subnet_ids: ['subnet-id1', 'subnet-id2']
"""

RETURN = r""" # """

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule

backoff_params = dict(retries=5, delay=1, backoff=1.5)


@AWSRetry.jittered_backoff(**backoff_params)
def describe_subnet_group(connection, subnet_group):
    """checks if instance exists"""
    try:
        subnet_group_filter = dict(Name="replication-subnet-group-id", Values=[subnet_group])
        return connection.describe_replication_subnet_groups(Filters=[subnet_group_filter])
    except botocore.exceptions.ClientError:
        return {"ReplicationSubnetGroups": []}


@AWSRetry.jittered_backoff(**backoff_params)
def replication_subnet_group_create(connection, **params):
    """creates the replication subnet group"""
    return connection.create_replication_subnet_group(**params)


@AWSRetry.jittered_backoff(**backoff_params)
def replication_subnet_group_modify(connection, **modify_params):
    return connection.modify_replication_subnet_group(**modify_params)


@AWSRetry.jittered_backoff(**backoff_params)
def replication_subnet_group_delete(module, connection):
    subnetid = module.params.get("identifier")
    delete_parameters = dict(ReplicationSubnetGroupIdentifier=subnetid)
    return connection.delete_replication_subnet_group(**delete_parameters)


def replication_subnet_exists(subnet):
    """Returns boolean based on the existence of the endpoint
    :param endpoint: dict containing the described endpoint
    :return: bool
    """
    return bool(len(subnet["ReplicationSubnetGroups"]))


def create_module_params(module):
    """
    Reads the module parameters and returns a dict
    :return: dict
    """
    instance_parameters = dict(
        # ReplicationSubnetGroupIdentifier gets translated to lower case anyway by the API
        ReplicationSubnetGroupIdentifier=module.params.get("identifier").lower(),
        ReplicationSubnetGroupDescription=module.params.get("description"),
        SubnetIds=module.params.get("subnet_ids"),
    )

    return instance_parameters


def compare_params(module, param_described):
    """
    Compares the dict obtained from the describe function and
    what we are reading from the values in the template We can
    never compare passwords as boto3's method for describing
    a DMS endpoint does not return the value for
    the password for security reasons ( I assume )
    """
    modparams = create_module_params(module)
    changed = False
    # need to sanitize values that get returned from the API
    if "VpcId" in param_described.keys():
        param_described.pop("VpcId")
    if "SubnetGroupStatus" in param_described.keys():
        param_described.pop("SubnetGroupStatus")
    for paramname in modparams.keys():
        if paramname in param_described.keys() and param_described.get(paramname) == modparams[paramname]:
            pass
        elif paramname == "SubnetIds":
            subnets = []
            for subnet in param_described.get("Subnets"):
                subnets.append(subnet.get("SubnetIdentifier"))
            for modulesubnet in modparams["SubnetIds"]:
                if modulesubnet in subnets:
                    pass
        else:
            changed = True
    return changed


def create_replication_subnet_group(module, connection):
    try:
        params = create_module_params(module)
        return replication_subnet_group_create(connection, **params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to create DMS replication subnet group.")


def modify_replication_subnet_group(module, connection):
    try:
        modify_params = create_module_params(module)
        return replication_subnet_group_modify(connection, **modify_params)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to Modify the DMS replication subnet group.")


def main():
    argument_spec = dict(
        state=dict(type="str", choices=["present", "absent"], default="present"),
        identifier=dict(type="str", required=True),
        description=dict(type="str", required=True),
        subnet_ids=dict(type="list", elements="str", required=True),
    )
    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)
    exit_message = None
    changed = False

    state = module.params.get("state")
    dmsclient = module.client("dms")
    subnet_group = describe_subnet_group(dmsclient, module.params.get("identifier"))
    if state == "present":
        if replication_subnet_exists(subnet_group):
            if compare_params(module, subnet_group["ReplicationSubnetGroups"][0]):
                if not module.check_mode:
                    exit_message = modify_replication_subnet_group(module, dmsclient)
                else:
                    exit_message = dmsclient
                changed = True
            else:
                exit_message = "No changes to Subnet group"
        else:
            if not module.check_mode:
                exit_message = create_replication_subnet_group(module, dmsclient)
                changed = True
            else:
                exit_message = "Check mode enabled"

    elif state == "absent":
        if replication_subnet_exists(subnet_group):
            if not module.check_mode:
                replication_subnet_group_delete(module, dmsclient)
                changed = True
                exit_message = "Replication subnet group Deleted"
            else:
                exit_message = dmsclient
                changed = True

        else:
            changed = False
            exit_message = "Replication subnet group does not exist"

    module.exit_json(changed=changed, msg=exit_message)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Michael DeHaan <michael.dehaan@gmail.com>, 2012-2013
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from ansible.module_utils.six import integer_types
from ansible.module_utils.six import string_types


def ansible_dict_to_boto3_filter_list(filters_dict):
    """Convert an Ansible dict of filters to list of dicts that boto3 can use
    Args:
        filters_dict (dict): Dict of AWS filters.
    Basic Usage:
        >>> filters = {'some-aws-id': 'i-01234567'}
        >>> ansible_dict_to_boto3_filter_list(filters)
        {
            'some-aws-id': 'i-01234567'
        }
    Returns:
        List: List of AWS filters and their values
        [
            {
                'Name': 'some-aws-id',
                'Values': [
                    'i-01234567',
                ]
            }
        ]
    """

    filters_list = []
    for k, v in filters_dict.items():
        filter_dict = {"Name": k}
        if isinstance(v, bool):
            filter_dict["Values"] = [str(v).lower()]
        elif isinstance(v, integer_types):
            filter_dict["Values"] = [str(v)]
        elif isinstance(v, string_types):
            filter_dict["Values"] = [v]
        else:
            filter_dict["Values"] = v

        filters_list.append(filter_dict)

    return filters_list


def map_complex_type(complex_type, type_map):
    """
    Allows to cast elements within a dictionary to a specific type
    Example of usage:

    DEPLOYMENT_CONFIGURATION_TYPE_MAP = {
        'maximum_percent': 'int',
        'minimum_healthy_percent': 'int'
    }

    deployment_configuration = map_complex_type(module.params['deployment_configuration'],
                                                DEPLOYMENT_CONFIGURATION_TYPE_MAP)

    This ensures all keys within the root element are casted and valid integers
    """

    if complex_type is None:
        return
    new_type = type(complex_type)()
    if isinstance(complex_type, dict):
        for key in complex_type:
            if key in type_map:
                if isinstance(type_map[key], list):
                    new_type[key] = map_complex_type(complex_type[key], type_map[key][0])
                else:
                    new_type[key] = map_complex_type(complex_type[key], type_map[key])
            else:
                new_type[key] = complex_type[key]
    elif isinstance(complex_type, list):
        for i in range(len(complex_type)):
            new_type.append(map_complex_type(complex_type[i], type_map))
    elif type_map:
        return globals()["__builtins__"][type_map](complex_type)
    return new_type


def scrub_none_parameters(parameters, descend_into_lists=True):
    """
    Iterate over a dictionary removing any keys that have a None value

    Reference: https://github.com/ansible-collections/community.aws/issues/251
    Credit: https://medium.com/better-programming/how-to-remove-null-none-values-from-a-dictionary-in-python-1bedf1aab5e4

    :param descend_into_lists: whether or not to descend in to lists to continue to remove None values
    :param parameters: parameter dict
    :return: parameter dict with all keys = None removed
    """

    clean_parameters = {}

    for k, v in parameters.items():
        if isinstance(v, dict):
            clean_parameters[k] = scrub_none_parameters(v, descend_into_lists=descend_into_lists)
        elif descend_into_lists and isinstance(v, list):
            clean_parameters[k] = [
                scrub_none_parameters(vv, descend_into_lists=descend_into_lists) if isinstance(vv, dict) else vv
                for vv in v
            ]
        elif v is not None:
            clean_parameters[k] = v

    return clean_parameters



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: organizations_appliance_vpn_third_party_vpnpeers
short_description: Resource module for organizations _appliance _vpn _thirdpartyvpnpeers
description:
- Manage operation update of the resource organizations _appliance _vpn _thirdpartyvpnpeers.
- Update the third party VPN peers for an organization.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  peers:
    description: The list of VPN peers.
    elements: dict
    suboptions:
      ikeVersion:
        description: Optional The IKE version to be used for the IPsec VPN peer configuration.
          Defaults to '1' when omitted.
        type: str
      ipsecPolicies:
        description: Custom IPSec policies for the VPN peer. If not included and a preset
          has not been chosen, the default preset for IPSec policies will be used.
        suboptions:
          childAuthAlgo:
            description: This is the authentication algorithms to be used in Phase 2.
              The value should be an array with one of the following algorithms 'sha256',
              'sha1', 'md5'.
            elements: str
            type: list
          childCipherAlgo:
            description: This is the cipher algorithms to be used in Phase 2. The value
              should be an array with one or more of the following algorithms 'aes256',
              'aes192', 'aes128', 'tripledes', 'des', 'null'.
            elements: str
            type: list
          childLifetime:
            description: The lifetime of the Phase 2 SA in seconds.
            type: int
          childPfsGroup:
            description: This is the Diffie-Hellman group to be used for Perfect Forward
              Secrecy in Phase 2. The value should be an array with one of the following
              values 'disabled','group14', 'group5', 'group2', 'group1'.
            elements: str
            type: list
          ikeAuthAlgo:
            description: This is the authentication algorithm to be used in Phase 1.
              The value should be an array with one of the following algorithms 'sha256',
              'sha1', 'md5'.
            elements: str
            type: list
          ikeCipherAlgo:
            description: This is the cipher algorithm to be used in Phase 1. The value
              should be an array with one of the following algorithms 'aes256', 'aes192',
              'aes128', 'tripledes', 'des'.
            elements: str
            type: list
          ikeDiffieHellmanGroup:
            description: This is the Diffie-Hellman group to be used in Phase 1. The
              value should be an array with one of the following algorithms 'group14',
              'group5', 'group2', 'group1'.
            elements: str
            type: list
          ikeLifetime:
            description: The lifetime of the Phase 1 SA in seconds.
            type: int
          ikePrfAlgo:
            description: Optional This is the pseudo-random function to be used in IKE_SA.
              The value should be an array with one of the following algorithms 'prfsha256',
              'prfsha1', 'prfmd5', 'default'. The 'default' option can be used to default
              to the Authentication algorithm.
            elements: str
            type: list
        type: dict
      ipsecPoliciesPreset:
        description: One of the following available presets 'default', 'aws', 'azure'.
          If this is provided, the 'ipsecPolicies' parameter is ignored.
        type: str
      localId:
        description: Optional The local ID is used to identify the MX to the peer. This
          will apply to all MXs this peer applies to.
        type: str
      name:
        description: The name of the VPN peer.
        type: str
      networkTags:
        description: A list of network tags that will connect with this peer. Use 'all'
          for all networks. Use 'none' for no networks. If not included, the default
          is 'all'.
        elements: str
        type: list
      privateSubnets:
        description: The list of the private subnets of the VPN peer.
        elements: str
        type: list
      publicIp:
        description: Optional The public IP of the VPN peer.
        type: str
      remoteId:
        description: Optional The remote ID is used to identify the connecting VPN peer.
          This can either be a valid IPv4 Address, FQDN or User FQDN.
        type: str
      secret:
        description: The shared secret with the VPN peer.
        type: str
    type: list
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for appliance updateOrganizationApplianceVpnThirdPartyVPNPeers
  description: Complete reference of the updateOrganizationApplianceVpnThirdPartyVPNPeers API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-organization-appliance-vpn-third-party-vpn-peers
notes:
  - SDK Method used are
    appliance.Appliance.update_organization_appliance_vpn_third_party_vpnpeers,

  - Paths used are
    put /organizations/{organizationId}/appliance/vpn/thirdPartyVPNPeers,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.organizations_appliance_vpn_third_party_vpnpeers:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    organizationId: string
    peers:
    - ikeVersion: '2'
      ipsecPolicies:
        childAuthAlgo:
        - sha1
        childCipherAlgo:
        - aes128
        childLifetime: 28800
        childPfsGroup:
        - disabled
        ikeAuthAlgo:
        - sha1
        ikeCipherAlgo:
        - tripledes
        ikeDiffieHellmanGroup:
        - group2
        ikeLifetime: 28800
        ikePrfAlgo:
        - prfsha1
      ipsecPoliciesPreset: default
      localId: myMXId@meraki.com
      name: Peer Name
      networkTags:
      - none
      privateSubnets:
      - 192.168.1.0/24
      - 192.168.128.0/24
      publicIp: 123.123.123.1
      remoteId: miles@meraki.com
      secret: Sample Password

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    [
      {
        "name": "string",
        "publicIp": "string",
        "remoteId": "string",
        "localId": "string",
        "secret": "string",
        "privateSubnets": [
          "string"
        ],
        "ipsecPolicies": {
          "ikeCipherAlgo": [
            "string"
          ],
          "ikeAuthAlgo": [
            "string"
          ],
          "ikePrfAlgo": [
            "string"
          ],
          "ikeDiffieHellmanGroup": [
            "string"
          ],
          "ikeLifetime": 0,
          "childCipherAlgo": [
            "string"
          ],
          "childAuthAlgo": [
            "string"
          ],
          "childPfsGroup": [
            "string"
          ],
          "childLifetime": 0
        },
        "ipsecPoliciesPreset": "string",
        "ikeVersion": "string",
        "networkTags": [
          "string"
        ]
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: autoscaling_launch_config
version_added: 1.0.0

short_description: Create or delete AWS Autoscaling Launch Configurations

description:
  - Can create or delete AWS Autoscaling Configurations.
  - Works with the M(community.aws.autoscaling_group) module to manage Autoscaling Groups.
  - Prior to release 5.0.0 this module was called C(community.aws.ec2_lc).
    The usage did not change.

notes:
  - Amazon ASG Autoscaling Launch Configurations are immutable once created, so modifying the configuration after it is changed will not modify the
    launch configuration on AWS. You must create a new config and assign it to the ASG instead.

author:
  - "Gareth Rushgrove (@garethr)"
  - "Willem van Ketwich (@wilvk)"

options:
  state:
    description:
      - Register or deregister the instance.
    default: present
    choices: ['present', 'absent']
    type: str
  name:
    description:
      - Unique name for configuration.
    required: true
    type: str
  instance_type:
    description:
      - Instance type to use for the instance.
      - Required when creating a new Launch Configuration.
    type: str
  image_id:
    description:
      - The AMI unique identifier to be used for the group.
    type: str
  key_name:
    description:
      - The SSH key name to be used for access to managed instances.
    type: str
  security_groups:
    description:
      - A list of security groups to apply to the instances.
      - You can specify either security group names or IDs or a mix.
    type: list
    elements: str
    default: []
  volumes:
    description:
      - A list dictionaries defining the volumes to create.
      - For any volume, a volume size less than C(1) will be interpreted as a request not to create the volume.
    type: list
    elements: dict
    suboptions:
      device_name:
        type: str
        description:
          - The name for the volume (For example C(/dev/sda)).
        required: true
      no_device:
        type: bool
        description:
          - When I(no_device=true) the device will not be created.
      snapshot:
        type: str
        description:
          - The ID of an EBS snapshot to copy when creating the volume.
          - Mutually exclusive with the I(ephemeral) parameter.
      ephemeral:
        type: str
        description:
          - Whether the volume should be ephemeral.
          - Data on ephemeral volumes is lost when the instance is stopped.
          - Mutually exclusive with the I(snapshot) parameter.
      volume_size:
        type: int
        description:
          - The size of the volume (in GiB).
          - Required unless one of I(ephemeral), I(snapshot) or I(no_device) is set.
      volume_type:
        type: str
        description:
          - The type of volume to create.
          - See
            U(https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSVolumeTypes.html) for more information on the available volume types.
      delete_on_termination:
        type: bool
        default: false
        description:
          - Whether the volume should be automatically deleted when the instance
            is terminated.
      iops:
        type: int
        description:
          - The number of IOPS per second to provision for the volume.
          - Required when I(volume_type=io1).
      throughput:
        type: int
        description:
          - The throughput to provision for a gp3 volume.
          - Valid Range is a minimum value of 125 and a maximum value of 1000.
        version_added: 3.1.0
      encrypted:
        type: bool
        default: false
        description:
          - Whether the volume should be encrypted using the 'aws/ebs' KMS CMK.
  user_data:
    description:
      - Opaque blob of data which is made available to the ec2 instance. Mutually exclusive with I(user_data_path).
    type: str
  user_data_path:
    description:
      - Path to the file that contains userdata for the ec2 instances. Mutually exclusive with I(user_data).
    type: path
  kernel_id:
    description:
      - Kernel id for the EC2 instance.
    type: str
  spot_price:
    description:
      - The spot price you are bidding. Only applies for an autoscaling group with spot instances.
    type: float
  instance_monitoring:
    description:
      - Specifies whether instances are launched with detailed monitoring.
    type: bool
    default: false
  assign_public_ip:
    description:
      - Used for Auto Scaling groups that launch instances into an Amazon Virtual Private Cloud. Specifies whether to assign a public IP address
        to each instance launched in a Amazon VPC.
    type: bool
  ramdisk_id:
    description:
      - A RAM disk id for the instances.
    type: str
  instance_profile_name:
    description:
      - The name or the Amazon Resource Name (ARN) of the instance profile associated with the IAM role for the instances.
    type: str
  ebs_optimized:
    description:
      - Specifies whether the instance is optimized for EBS I/O (true) or not (false).
    default: false
    type: bool
  classic_link_vpc_id:
    description:
      - Id of ClassicLink enabled VPC
    type: str
  classic_link_vpc_security_groups:
    description:
      - A list of security group IDs with which to associate the ClassicLink VPC instances.
    type: list
    elements: str
  vpc_id:
    description:
      - VPC ID, used when resolving security group names to IDs.
    type: str
  instance_id:
    description:
      - The Id of a running instance to use as a basis for a launch configuration. Can be used in place of I(image_id) and I(instance_type).
    type: str
  placement_tenancy:
    description:
      - Determines whether the instance runs on single-tenant hardware or not.
      - When not set AWS will default to C(default).
    type: str
    choices: ['default', 'dedicated']
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""

- name: create a launch configuration with an encrypted volume
  community.aws.autoscaling_launch_config:
    name: special
    image_id: ami-XXX
    key_name: default
    security_groups:
      - 'group'
      - 'group2'
    instance_type: t1.micro
    volumes:
      - device_name: /dev/sda1
        volume_size: 100
        volume_type: io1
        iops: 3000
        delete_on_termination: true
        encrypted: true
      - device_name: /dev/sdb
        ephemeral: ephemeral0

- name: create a launch configuration using a running instance id as a basis
  community.aws.autoscaling_launch_config:
    name: special
    instance_id: i-00a48b207ec59e948
    key_name: default
    security_groups:
      - 'launch-wizard-2'
    volumes:
      - device_name: /dev/sda1
        volume_size: 120
        volume_type: io1
        iops: 3000
        delete_on_termination: true

- name: create a launch configuration to omit the /dev/sdf EBS device that is included in the AMI image
  community.aws.autoscaling_launch_config:
    name: special
    image_id: ami-XXX
    key_name: default
    security_groups:
      - 'group'
      - 'group2'
    instance_type: t1.micro
    volumes:
      - device_name: /dev/sdf
        no_device: true

- name: Use EBS snapshot ID for volume
  block:
    - name: Set Volume Facts
      ansible.builtin.set_fact:
        volumes:
          - device_name: /dev/sda1
            volume_size: 20
            ebs:
              snapshot: snap-XXXX
              volume_type: gp2
              delete_on_termination: true
              encrypted: false

    - name: Create launch configuration
      community.aws.autoscaling_launch_config:
        name: lc1
        image_id: ami-xxxx
        assign_public_ip: true
        instance_type: t2.medium
        key_name: my-key
        security_groups:
          - 'sg-xxxx'
        volumes: "{{ volumes }}"
      register: lc_info
"""

RETURN = r"""
arn:
  description: The Amazon Resource Name of the launch configuration.
  returned: when I(state=present)
  type: str
  sample: arn:aws:autoscaling:us-east-1:123456789012:launchConfiguration:888d9b58-d93a-40c4-90cf-759197a2621a:launchConfigurationName/launch_config_name
changed:
  description: Whether the state of the launch configuration has changed.
  returned: always
  type: bool
  sample: false
created_time:
  description: The creation date and time for the launch configuration.
  returned: when I(state=present)
  type: str
  sample: '2017-11-03 23:46:44.841000'
image_id:
  description: The ID of the Amazon Machine Image used by the launch configuration.
  returned: when I(state=present)
  type: str
  sample: ami-9be6f38c
instance_type:
  description: The instance type for the instances.
  returned: when I(state=present)
  type: str
  sample: t1.micro
name:
  description: The name of the launch configuration.
  returned: when I(state=present)
  type: str
  sample: launch_config_name
result:
  description: The specification details for the launch configuration.
  returned: when I(state=present)
  type: complex
  contains:
    PlacementTenancy:
      description: The tenancy of the instances, either default or dedicated.
      returned: when I(state=present)
      type: str
      sample: default
    associate_public_ip_address:
      description: (EC2-VPC) Indicates whether to assign a public IP address to each instance.
      returned: when I(state=present)
      type: bool
      sample: false
    block_device_mappings:
      description: A block device mapping, which specifies the block devices.
      returned: when I(state=present)
      type: complex
      contains:
        device_name:
          description: The device name exposed to the EC2 instance (for example, /dev/sdh or xvdh).
          returned: when I(state=present)
          type: str
          sample: /dev/sda1
        ebs:
          description: The information about the Amazon EBS volume.
          returned: when I(state=present)
          type: complex
          contains:
            snapshot_id:
              description: The ID of the snapshot.
              returned: when I(state=present)
              type: str
            volume_size:
              description: The volume size, in GiB.
              returned: when I(state=present)
              type: str
              sample: '100'
        virtual_name:
          description: The name of the virtual device (for example, ephemeral0).
          returned: when I(state=present)
          type: str
          sample: ephemeral0
    classic_link_vpc_id:
      description: The ID of a ClassicLink-enabled VPC to link your EC2-Classic instances to.
      returned: when I(state=present)
      type: str
    classic_link_vpc_security_groups:
      description: The IDs of one or more security groups for the VPC specified in ClassicLinkVPCId.
      returned: when I(state=present)
      type: list
      sample: []
    created_time:
      description: The creation date and time for the launch configuration.
      returned: when I(state=present)
      type: str
      sample: '2017-11-03 23:46:44.841000'
    delete_on_termination:
      description: Indicates whether the volume is deleted on instance termination.
      returned: when I(state=present)
      type: bool
      sample: true
    ebs_optimized:
      description: Indicates whether the instance is optimized for EBS I/O C(true) or not C(false).
      returned: when I(state=present)
      type: bool
      sample: false
    image_id:
      description: The ID of the Amazon Machine Image used by the launch configuration.
      returned: when I(state=present)
      type: str
      sample: ami-9be6f38c
    instance_monitoring:
      description: Indicates whether instances in this group are launched with detailed C(true) or basic C(false) monitoring.
      returned: when I(state=present)
      type: bool
      sample: true
    instance_profile_name:
      description: The name or Amazon Resource Name (ARN) of the instance profile associated with the IAM role for the instance.
      returned: when I(state=present)
      type: str
      sample: null
    instance_type:
      description: The instance type for the instances.
      returned: when I(state=present)
      type: str
      sample: t1.micro
    iops:
      description: The number of I/O operations per second (IOPS) to provision for the volume.
      returned: when I(state=present)
      type: int
    kernel_id:
      description: The ID of the kernel associated with the AMI.
      returned: when I(state=present)
      type: str
      sample: ''
    key_name:
      description: The name of the key pair.
      returned: when I(state=present)
      type: str
      sample: testkey
    launch_configuration_arn:
      description: The Amazon Resource Name (ARN) of the launch configuration.
      returned: when I(state=present)
      type: str
      sample: arn:aws:autoscaling:us-east-1:123456789012:launchConfiguration:888d9b58-d93a-40c4-90cf-759197a2621a:launchConfigurationName/launch_config_name
    member:
      description: ""
      returned: when I(state=present)
      type: str
      sample: "\n      "
    name:
      description: The name of the launch configuration.
      returned: when I(state=present)
      type: str
      sample: launch_config_name
    ramdisk_id:
      description: The ID of the RAM disk associated with the AMI.
      returned: when I(state=present)
      type: str
      sample: ''
    security_groups:
      description: The security groups to associate with the instances.
      returned: when I(state=present)
      type: list
      sample:
      - sg-5e27db2f
    spot_price:
      description: The price to bid when launching Spot Instances.
      returned: when I(state=present)
      type: float
    use_block_device_types:
      description: Indicates whether to suppress a device mapping.
      returned: when I(state=present)
      type: bool
      sample: false
    user_data:
      description: The user data available to the instances.
      returned: when I(state=present)
      type: str
      sample: ''
    volume_type:
      description: The volume type (one of standard, io1, gp2).
      returned: when I(state=present)
      type: str
      sample: io1
security_groups:
  description: The security groups to associate with the instances.
  returned: when I(state=present)
  type: list
  sample:
  - sg-5e27db2f
"""


import traceback

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils._text import to_text
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import get_ec2_security_group_ids_from_names

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def create_block_device_meta(module, volume):
    if "snapshot" not in volume and "ephemeral" not in volume and "no_device" not in volume:
        if "volume_size" not in volume:
            module.fail_json(msg="Size must be specified when creating a new volume or modifying the root volume")
    if "snapshot" in volume:
        if volume.get("volume_type") == "io1" and "iops" not in volume:
            module.fail_json(msg="io1 volumes must have an iops value set")
    if "ephemeral" in volume:
        if "snapshot" in volume:
            module.fail_json(msg="Cannot set both ephemeral and snapshot")

    return_object = {}

    if "ephemeral" in volume:
        return_object["VirtualName"] = volume.get("ephemeral")

    if "device_name" in volume:
        return_object["DeviceName"] = volume.get("device_name")

    if "no_device" in volume:
        return_object["NoDevice"] = volume.get("no_device")

    if any(
        key in volume
        for key in [
            "snapshot",
            "volume_size",
            "volume_type",
            "delete_on_termination",
            "iops",
            "throughput",
            "encrypted",
        ]
    ):
        return_object["Ebs"] = {}

    if "snapshot" in volume:
        return_object["Ebs"]["SnapshotId"] = volume.get("snapshot")

    if "volume_size" in volume:
        return_object["Ebs"]["VolumeSize"] = int(volume.get("volume_size", 0))

    if "volume_type" in volume:
        return_object["Ebs"]["VolumeType"] = volume.get("volume_type")

    if "delete_on_termination" in volume:
        return_object["Ebs"]["DeleteOnTermination"] = volume.get("delete_on_termination", False)

    if "iops" in volume:
        return_object["Ebs"]["Iops"] = volume.get("iops")

    if "throughput" in volume:
        if volume.get("volume_type") != "gp3":
            module.fail_json(msg="The throughput parameter is supported only for GP3 volumes.")
        return_object["Ebs"]["Throughput"] = volume.get("throughput")

    if "encrypted" in volume:
        return_object["Ebs"]["Encrypted"] = volume.get("encrypted")

    return return_object


def create_launch_config(connection, module):
    name = module.params.get("name")
    vpc_id = module.params.get("vpc_id")
    try:
        ec2_connection = module.client("ec2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")
    try:
        security_groups = get_ec2_security_group_ids_from_names(
            module.params.get("security_groups"), ec2_connection, vpc_id=vpc_id, boto3=True
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to get Security Group IDs")
    except ValueError as e:
        module.fail_json(msg="Failed to get Security Group IDs", exception=traceback.format_exc())
    user_data = module.params.get("user_data")
    user_data_path = module.params.get("user_data_path")
    volumes = module.params["volumes"]
    instance_monitoring = module.params.get("instance_monitoring")
    assign_public_ip = module.params.get("assign_public_ip")
    instance_profile_name = module.params.get("instance_profile_name")
    ebs_optimized = module.params.get("ebs_optimized")
    classic_link_vpc_id = module.params.get("classic_link_vpc_id")
    classic_link_vpc_security_groups = module.params.get("classic_link_vpc_security_groups")

    block_device_mapping = []

    convert_list = [
        "image_id",
        "instance_type",
        "instance_type",
        "instance_id",
        "placement_tenancy",
        "key_name",
        "kernel_id",
        "ramdisk_id",
        "spot_price",
    ]

    launch_config = snake_dict_to_camel_dict(
        dict((k.capitalize(), str(v)) for k, v in module.params.items() if v is not None and k in convert_list)
    )

    if user_data_path:
        try:
            with open(user_data_path, "r") as user_data_file:
                user_data = user_data_file.read()
        except IOError as e:
            module.fail_json(msg="Failed to open file for reading", exception=traceback.format_exc())

    if volumes:
        for volume in volumes:
            if "device_name" not in volume:
                module.fail_json(msg="Device name must be set for volume")
            # Minimum volume size is 1GiB. We'll use volume size explicitly set to 0 to be a signal not to create this volume
            if "volume_size" not in volume or int(volume["volume_size"]) > 0:
                block_device_mapping.append(create_block_device_meta(module, volume))

    try:
        launch_configs = connection.describe_launch_configurations(LaunchConfigurationNames=[name]).get(
            "LaunchConfigurations"
        )
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to describe launch configuration by name")

    changed = False
    result = {}

    launch_config["LaunchConfigurationName"] = name

    if security_groups is not None:
        launch_config["SecurityGroups"] = security_groups

    if classic_link_vpc_id is not None:
        launch_config["ClassicLinkVPCId"] = classic_link_vpc_id

    if instance_monitoring is not None:
        launch_config["InstanceMonitoring"] = {"Enabled": instance_monitoring}

    if classic_link_vpc_security_groups is not None:
        launch_config["ClassicLinkVPCSecurityGroups"] = classic_link_vpc_security_groups

    if block_device_mapping:
        launch_config["BlockDeviceMappings"] = block_device_mapping

    if instance_profile_name is not None:
        launch_config["IamInstanceProfile"] = instance_profile_name

    if assign_public_ip is not None:
        launch_config["AssociatePublicIpAddress"] = assign_public_ip

    if user_data is not None:
        launch_config["UserData"] = user_data

    if ebs_optimized is not None:
        launch_config["EbsOptimized"] = ebs_optimized

    if len(launch_configs) == 0:
        try:
            connection.create_launch_configuration(**launch_config)
            launch_configs = connection.describe_launch_configurations(LaunchConfigurationNames=[name]).get(
                "LaunchConfigurations"
            )
            changed = True
            if launch_configs:
                launch_config = launch_configs[0]
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to create launch configuration")

    result = dict(
        (k, v)
        for k, v in launch_config.items()
        if k not in ["Connection", "CreatedTime", "InstanceMonitoring", "BlockDeviceMappings"]
    )

    result["CreatedTime"] = to_text(launch_config.get("CreatedTime"))

    try:
        result["InstanceMonitoring"] = module.boolean(launch_config.get("InstanceMonitoring").get("Enabled"))
    except AttributeError:
        result["InstanceMonitoring"] = False

    result["BlockDeviceMappings"] = []

    for block_device_mapping in launch_config.get("BlockDeviceMappings", []):
        result["BlockDeviceMappings"].append(
            dict(
                device_name=block_device_mapping.get("DeviceName"), virtual_name=block_device_mapping.get("VirtualName")
            )
        )
        if block_device_mapping.get("Ebs") is not None:
            result["BlockDeviceMappings"][-1]["ebs"] = dict(
                snapshot_id=block_device_mapping.get("Ebs").get("SnapshotId"),
                volume_size=block_device_mapping.get("Ebs").get("VolumeSize"),
            )

    if user_data_path:
        result["UserData"] = "hidden"  # Otherwise, we dump binary to the user's terminal

    return_object = {
        "Name": result.get("LaunchConfigurationName"),
        "CreatedTime": result.get("CreatedTime"),
        "ImageId": result.get("ImageId"),
        "Arn": result.get("LaunchConfigurationARN"),
        "SecurityGroups": result.get("SecurityGroups"),
        "InstanceType": result.get("InstanceType"),
        "Result": result,
    }

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(return_object))


def delete_launch_config(connection, module):
    try:
        name = module.params.get("name")
        launch_configs = connection.describe_launch_configurations(LaunchConfigurationNames=[name]).get(
            "LaunchConfigurations"
        )
        if launch_configs:
            connection.delete_launch_configuration(
                LaunchConfigurationName=launch_configs[0].get("LaunchConfigurationName")
            )
            module.exit_json(changed=True)
        else:
            module.exit_json(changed=False)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to delete launch configuration")


def main():
    argument_spec = dict(
        name=dict(required=True),
        image_id=dict(),
        instance_id=dict(),
        key_name=dict(),
        security_groups=dict(default=[], type="list", elements="str"),
        user_data=dict(),
        user_data_path=dict(type="path"),
        kernel_id=dict(),
        volumes=dict(type="list", elements="dict"),
        instance_type=dict(),
        state=dict(default="present", choices=["present", "absent"]),
        spot_price=dict(type="float"),
        ramdisk_id=dict(),
        instance_profile_name=dict(),
        ebs_optimized=dict(default=False, type="bool"),
        instance_monitoring=dict(default=False, type="bool"),
        assign_public_ip=dict(type="bool"),
        classic_link_vpc_security_groups=dict(type="list", elements="str"),
        classic_link_vpc_id=dict(),
        vpc_id=dict(),
        placement_tenancy=dict(choices=["default", "dedicated"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["user_data", "user_data_path"]],
    )

    try:
        connection = module.client("autoscaling")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="unable to establish connection")

    state = module.params.get("state")

    if state == "present":
        create_launch_config(connection, module)
    elif state == "absent":
        delete_launch_config(connection, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: byod_portal
short_description: Resource module for BYOD Portal
description:
- Manage operations create, update and delete of the resource BYOD Portal.
- This API creates a BYOD portal.
- This API deletes a BYOD portal by ID.
- This API allows the client to update a BYOD portal by ID.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  customizations:
    description: Defines all of the Portal Customizations available for a BYOD.
    suboptions:
      globalCustomizations:
        description: Represent the portal Global customizations.
        suboptions:
          backgroundImage:
            description: BYOD Portal's backgroundImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
          bannerImage:
            description: BYOD Portal's bannerImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
          bannerTitle:
            description: BYOD Portal's bannerTitle.
            type: str
          contactText:
            description: BYOD Portal's contactText.
            type: str
          desktopLogoImage:
            description: BYOD Portal's desktopLogoImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
          footerElement:
            description: BYOD Portal's footerElement.
            type: str
          mobileLogoImage:
            description: BYOD Portal's mobileLogoImage.
            suboptions:
              data:
                description: Represented as base 64 encoded string of the image byte
                  array.
                type: str
            type: dict
        type: dict
      language:
        description: This property is supported only for Read operation and it allows
          to show the customizations in English. Other languages are not supported.
        suboptions:
          viewLanguage:
            description: BYOD Portal's viewLanguage.
            type: str
        type: dict
      pageCustomizations:
        description: Represent the entire page customization as a giant dictionary.
        suboptions:
          data:
            description: The Dictionary will be exposed here as key value pair.
            elements: dict
            suboptions:
              key:
                description: BYOD Portal's key.
                type: str
              value:
                description: BYOD Portal's value.
                type: str
            type: list
        type: dict
      portalTheme:
        description: Defines the configuration for portal theme.
        suboptions:
          id:
            description: The unique internal identifier of the portal theme.
            type: str
          name:
            description: The system- or user-assigned name of the portal theme.
            type: str
          themeData:
            description: A CSS file, represented as a Base64-encoded byte array.
            type: str
        type: dict
      portalTweakSettings:
        description: The Tweak Settings are a customization of the Portal Theme that
          has been selected for the portal. When the Portal Theme selection is changed,
          the Tweak Settings are overwritten to match the values in the theme. The Tweak
          Settings can subsequently be changed by the user.
        suboptions:
          bannerColor:
            description: Hex value of color.
            type: str
          bannerTextColor:
            description: BYOD Portal's bannerTextColor.
            type: str
          pageBackgroundColor:
            description: BYOD Portal's pageBackgroundColor.
            type: str
          pageLabelAndTextColor:
            description: BYOD Portal's pageLabelAndTextColor.
            type: str
        type: dict
    type: dict
  description:
    description: BYOD Portal's description.
    type: str
  id:
    description: Resource UUID, mandatory for update.
    type: str
  name:
    description: Resource Name.
    type: str
  portalTestUrl:
    description: URL to bring up a test page for this portal.
    type: str
  portalType:
    description: Allowed values - BYOD, - HOTSPOTGUEST, - MYDEVICE, - SELFREGGUEST,
      - SPONSOR, - SPONSOREDGUEST.
    type: str
  settings:
    description: Defines all of the settings groups available for a BYOD.
    suboptions:
      byodSettings:
        description: Configuration of BYOD Device Welcome, Registration and Success
          steps.
        suboptions:
          byodRegistrationSettings:
            description: BYOD Portal's byodRegistrationSettings.
            suboptions:
              endPointIdentityGroupId:
                description: BYOD Portal's endPointIdentityGroupId.
                type: str
              showDeviceID:
                description: ShowDeviceID flag.
                type: bool
            type: dict
          byodRegistrationSuccessSettings:
            description: BYOD Portal's byodRegistrationSuccessSettings.
            suboptions:
              redirectUrl:
                description: Target URL for redirection, used when successRedirect =
                  URL.
                type: str
              successRedirect:
                description: After an Authentication Success where should device be
                  redirected. Allowed values.
                type: str
            type: dict
          byodWelcomeSettings:
            description: Configuration of BYOD endpoint welcome step configuration.
            suboptions:
              aupDisplay:
                description: How the AUP should be displayed, either on page or as a
                  link. Only valid if includeAup = true. Allowed values - ONPAGE, -
                  ASLINK.
                type: str
              enableBYOD:
                description: EnableBYOD flag.
                type: bool
              enableGuestAccess:
                description: EnableGuestAccess flag.
                type: bool
              includeAup:
                description: IncludeAup flag.
                type: bool
              requireAupAcceptance:
                description: RequireAupAcceptance flag.
                type: bool
              requireMDM:
                description: RequireMDM flag.
                type: bool
              requireScrolling:
                description: Require BYOD devices to scroll down to the bottom of the
                  AUP, Only valid if includeAup = true.
                type: bool
            type: dict
        type: dict
      portalSettings:
        description: The port, interface, certificate, and other basic settings of a
          portal.
        suboptions:
          allowedInterfaces:
            description: Interfaces that the portal will be reachable on. Allowed values
              - eth0, - eth1, - eth2, - eth3, - eth4, - eth5, - bond0, - bond1, - bond2.
            elements: str
            type: list
          alwaysUsedLanguage:
            description: Used when displayLang = ALWAYSUSE.
            type: str
          certificateGroupTag:
            description: Logical name of the x.509 server certificate that will be used
              for the portal.
            type: str
          displayLang:
            description: Allowed values - USEBROWSERLOCALE, - ALWAYSUSE.
            type: str
          endpointIdentityGroup:
            description: Unique Id of the endpoint identity group where user's devices
              will be added. Used only in Hotspot Portal.
            type: str
          fallbackLanguage:
            description: Used when displayLang = USEBROWSERLOCALE.
            type: str
          httpsPort:
            description: The port number that the allowed interfaces will listen on.
              Range from 8000 to 8999.
            type: int
        type: dict
      supportInfoSettings:
        description: BYOD Portal's supportInfoSettings.
        suboptions:
          defaultEmptyFieldValue:
            description: The default value displayed for an empty field. Only valid
              when emptyFieldDisplay = DISPLAYWITHDEFAULTVALUE.
            type: str
          emptyFieldDisplay:
            description: Specifies how empty fields are handled on the Support Information
              Page. Allowed values - HIDE, - DISPLAYWITHNOVALUE, - DISPLAYWITHDEFAULTVALUE.
            type: str
          includeBrowserUserAgent:
            description: IncludeBrowserUserAgent flag.
            type: bool
          includeFailureCode:
            description: IncludeFailureCode flag.
            type: bool
          includeIpAddress:
            description: IncludeIpAddress flag.
            type: bool
          includeMacAddr:
            description: IncludeMacAddr flag.
            type: bool
          includePolicyServer:
            description: IncludePolicyServer flag.
            type: bool
          includeSupportInfoPage:
            description: IncludeSupportInfoPage flag.
            type: bool
        type: dict
    type: dict
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    byod_portal.ByodPortal.create_byod_portal,
    byod_portal.ByodPortal.delete_byod_portal_by_id,
    byod_portal.ByodPortal.update_byod_portal_by_id,

  - Paths used are
    post /ers/config/byodportal,
    delete /ers/config/byodportal/{id},
    put /ers/config/byodportal/{id},

"""

EXAMPLES = r"""
- name: Update by id
  cisco.ise.byod_portal:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customizations:
      globalCustomizations:
        backgroundImage:
          data: string
        bannerImage:
          data: string
        bannerTitle: string
        contactText: string
        desktopLogoImage:
          data: string
        footerElement: string
        mobileLogoImage:
          data: string
      language:
        viewLanguage: string
      pageCustomizations:
        data:
        - key: string
          value: string
      portalTheme:
        id: string
        name: string
        themeData: string
      portalTweakSettings:
        bannerColor: string
        bannerTextColor: string
        pageBackgroundColor: string
        pageLabelAndTextColor: string
    description: string
    id: string
    name: string
    portalTestUrl: string
    portalType: string
    settings:
      byodSettings:
        byodRegistrationSettings:
          endPointIdentityGroupId: string
          showDeviceID: true
        byodRegistrationSuccessSettings:
          redirectUrl: string
          successRedirect: string
        byodWelcomeSettings:
          aupDisplay: string
          enableBYOD: true
          enableGuestAccess: true
          includeAup: true
          requireAupAcceptance: true
          requireMDM: true
          requireScrolling: true
      portalSettings:
        allowedInterfaces:
        - string
        alwaysUsedLanguage: string
        certificateGroupTag: string
        displayLang: string
        endpointIdentityGroup: string
        fallbackLanguage: string
        httpsPort: 0
      supportInfoSettings:
        defaultEmptyFieldValue: string
        emptyFieldDisplay: string
        includeBrowserUserAgent: true
        includeFailureCode: true
        includeIpAddress: true
        includeMacAddr: true
        includePolicyServer: true
        includeSupportInfoPage: true

- name: Delete by id
  cisco.ise.byod_portal:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    id: string

- name: Create
  cisco.ise.byod_portal:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    customizations:
      globalCustomizations:
        backgroundImage:
          data: string
        bannerImage:
          data: string
        bannerTitle: string
        contactText: string
        desktopLogoImage:
          data: string
        footerElement: string
        mobileLogoImage:
          data: string
      language:
        viewLanguage: string
      pageCustomizations:
        data:
        - key: string
          value: string
      portalTheme:
        id: string
        name: string
        themeData: string
      portalTweakSettings:
        bannerColor: string
        bannerTextColor: string
        pageBackgroundColor: string
        pageLabelAndTextColor: string
    description: string
    id: string
    name: string
    portalTestUrl: string
    portalType: string
    settings:
      byodSettings:
        byodRegistrationSettings:
          endPointIdentityGroupId: string
          showDeviceID: true
        byodRegistrationSuccessSettings:
          redirectUrl: string
          successRedirect: string
        byodWelcomeSettings:
          aupDisplay: string
          enableBYOD: true
          enableGuestAccess: true
          includeAup: true
          requireAupAcceptance: true
          requireMDM: true
          requireScrolling: true
      portalSettings:
        allowedInterfaces:
        - string
        alwaysUsedLanguage: string
        certificateGroupTag: string
        displayLang: string
        endpointIdentityGroup: string
        fallbackLanguage: string
        httpsPort: 0
      supportInfoSettings:
        defaultEmptyFieldValue: string
        emptyFieldDisplay: string
        includeBrowserUserAgent: true
        includeFailureCode: true
        includeIpAddress: true
        includeMacAddr: true
        includePolicyServer: true
        includeSupportInfoPage: true

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "portalType": "string",
      "portalTestUrl": "string",
      "settings": {
        "portalSettings": {
          "httpsPort": 0,
          "allowedInterfaces": [
            "string"
          ],
          "certificateGroupTag": "string",
          "endpointIdentityGroup": "string",
          "displayLang": "string",
          "fallbackLanguage": "string",
          "alwaysUsedLanguage": "string"
        },
        "byodSettings": {
          "byodWelcomeSettings": {
            "enableBYOD": true,
            "enableGuestAccess": true,
            "requireMDM": true,
            "includeAup": true,
            "aupDisplay": "string",
            "requireAupAcceptance": true,
            "requireScrolling": true
          },
          "byodRegistrationSettings": {
            "showDeviceID": true,
            "endPointIdentityGroupId": "string"
          },
          "byodRegistrationSuccessSettings": {
            "successRedirect": "string",
            "redirectUrl": "string"
          }
        },
        "supportInfoSettings": {
          "includeSupportInfoPage": true,
          "includeMacAddr": true,
          "includeIpAddress": true,
          "includeBrowserUserAgent": true,
          "includePolicyServer": true,
          "includeFailureCode": true,
          "emptyFieldDisplay": "string",
          "defaultEmptyFieldValue": "string"
        }
      },
      "customizations": {
        "portalTheme": {
          "id": "string",
          "name": "string",
          "themeData": "string"
        },
        "portalTweakSettings": {
          "bannerColor": "string",
          "bannerTextColor": "string",
          "pageBackgroundColor": "string",
          "pageLabelAndTextColor": "string"
        },
        "language": {
          "viewLanguage": "string"
        },
        "globalCustomizations": {
          "mobileLogoImage": {
            "data": "string"
          },
          "desktopLogoImage": {
            "data": "string"
          },
          "bannerImage": {
            "data": "string"
          },
          "backgroundImage": {
            "data": "string"
          },
          "bannerTitle": "string",
          "contactText": "string",
          "footerElement": "string"
        },
        "pageCustomizations": {
          "data": [
            {
              "key": "string",
              "value": "string"
            }
          ]
        }
      },
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_update_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: dict
  sample: >
    {
      "UpdatedFieldsList": {
        "updatedField": [
          {
            "field": "string",
            "oldValue": "string",
            "newValue": "string"
          }
        ],
        "field": "string",
        "oldValue": "string",
        "newValue": "string"
      }
    }
"""



# -*- coding: utf-8 -*-

# This file is part of Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


try:
    import botocore
except ImportError:
    pass  # Handled by HAS_BOTO3

from typing import Union

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict


def get_backup_resource_tags(module, backup_client, resource):
    try:
        response = backup_client.list_tags(ResourceArn=resource)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Failed to list tags on the resource {resource}")

    return response["Tags"]


def _list_backup_plans(client, backup_plan_name):
    first_iteration = False
    next_token = None

    # We can not use the paginator at the moment because if was introduced after boto3 version 1.22
    # paginator = client.get_paginator("list_backup_plans")
    # result = paginator.paginate(**params).build_full_result()["BackupPlansList"]

    response = client.list_backup_plans()
    next_token = response.get("NextToken", None)

    if next_token is None:
        entries = response["BackupPlansList"]
        for backup_plan in entries:
            if backup_plan_name == backup_plan["BackupPlanName"]:
                return backup_plan["BackupPlanId"]

    while next_token is not None:
        if first_iteration:
            response = client.list_backup_plans(NextToken=next_token)
        first_iteration = True
        entries = response["BackupPlansList"]
        for backup_plan in entries:
            if backup_plan_name == backup_plan["BackupPlanName"]:
                return backup_plan["BackupPlanId"]
        next_token = response.get("NextToken")


def get_plan_details(module, client, backup_plan_name: str):
    backup_plan_id = _list_backup_plans(client, backup_plan_name)

    if not backup_plan_id:
        return []

    try:
        result = client.get_backup_plan(BackupPlanId=backup_plan_id)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Failed to describe plan {backup_plan_id}")

    # Turn the boto3 result in to ansible_friendly_snaked_names
    snaked_backup_plan = []

    try:
        resource = result.get("BackupPlanArn", None)
        tag_dict = get_backup_resource_tags(module, client, resource)
        result.update({"tags": tag_dict})
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Failed to get the backup plan tags")

    snaked_backup_plan.append(camel_dict_to_snake_dict(result, ignore_list="tags"))

    # Remove AWS API response and add top-level plan name
    for v in snaked_backup_plan:
        if "response_metadata" in v:
            del v["response_metadata"]
        v["backup_plan_name"] = v["backup_plan"]["backup_plan_name"]

    return snaked_backup_plan


def _list_backup_selections(client, module, plan_id):
    first_iteration = False
    next_token = None
    selections = []

    # We can not use the paginator at the moment because if was introduced after boto3 version 1.22
    # paginator = client.get_paginator("list_backup_selections")
    # result = paginator.paginate(**params).build_full_result()["BackupSelectionsList"]

    try:
        response = client.list_backup_selections(BackupPlanId=plan_id)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to list AWS backup selections")

    next_token = response.get("NextToken", None)

    if next_token is None:
        return response["BackupSelectionsList"]

    while next_token:
        if first_iteration:
            try:
                response = client.list_backup_selections(BackupPlanId=plan_id, NextToken=next_token)
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                module.fail_json_aws(e, msg="Failed to list AWS backup selections")
        first_iteration = True
        selections.append(response["BackupSelectionsList"])
        next_token = response.get("NextToken")


def _get_backup_selection(client, module, plan_id, selection_id):
    try:
        result = client.get_backup_selection(BackupPlanId=plan_id, SelectionId=selection_id)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Failed to describe selection {selection_id}")
    return result or []


def get_selection_details(module, client, plan_name: str, selection_name: Union[str, list]):
    result = []

    plan = get_plan_details(module, client, plan_name)

    if not plan:
        module.fail_json(msg=f"The backup plan {plan_name} does not exist. Please create one first.")

    plan_id = plan[0]["backup_plan_id"]

    selection_list = _list_backup_selections(client, module, plan_id)

    if selection_name:
        for selection in selection_list:
            if isinstance(selection_name, list):
                for name in selection_name:
                    if selection["SelectionName"] == name:
                        selection_id = selection["SelectionId"]
                        selection_info = _get_backup_selection(client, module, plan_id, selection_id)
                        result.append(selection_info)
            if isinstance(selection_name, str):
                if selection["SelectionName"] == selection_name:
                    selection_id = selection["SelectionId"]
                    result.append(_get_backup_selection(client, module, plan_id, selection_id))
                    break
    else:
        for selection in selection_list:
            selection_id = selection["SelectionId"]
            result.append(_get_backup_selection(client, module, plan_id, selection_id))

    for v in result:
        if "ResponseMetadata" in v:
            del v["ResponseMetadata"]
        if "BackupSelection" in v:
            for backup_selection_key in v["BackupSelection"]:
                v[backup_selection_key] = v["BackupSelection"][backup_selection_key]
        del v["BackupSelection"]

    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: wafv2_web_acl_info
version_added: 1.5.0
author:
  - "Markus Bergholz (@markuman)"
short_description: wafv2_web_acl
description:
  - Info about web acl
options:
  name:
    description:
      - The name of the web acl.
    required: true
    type: str
  scope:
    description:
      - Scope of wafv2 web acl.
    required: true
    choices: ["CLOUDFRONT", "REGIONAL"]
    type: str

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: get web acl
  community.aws.wafv2_web_acl_info:
    name: test05
    scope: REGIONAL
  register: out
"""

RETURN = r"""
arn:
  description: web acl arn
  sample: arn:aws:wafv2:eu-central-1:11111111:regional/webacl/test05/318c1ab9-fa74-4b3b-a974-f92e25106f61
  type: str
  returned: Always, as long as the web acl exists
description:
  description: Description of the web acl
  sample: Some web acl description
  returned: Always, as long as the web acl exists
  type: str
capacity:
  description: Current capacity of the web acl
  sample: 140
  returned: Always, as long as the web acl exists
  type: int
name:
  description: Web acl name
  sample: test02
  returned: Always, as long as the web acl exists
  type: str
rules:
  description: Current rules of the web acl
  returned: Always, as long as the web acl exists
  type: list
  sample:
    - name: admin_protect
      override_action:
        none: {}
      priority: 1
      statement:
        managed_rule_group_statement:
          name: AWSManagedRulesAdminProtectionRuleSet
          vendor_name: AWS
      visibility_config:
        cloud_watch_metrics_enabled: true
        metric_name: admin_protect
        sampled_requests_enabled: true
visibility_config:
  description: Visibility config of the web acl
  returned: Always, as long as the web acl exists
  type: dict
  sample:
    cloud_watch_metrics_enabled: true
    metric_name: blub
    sampled_requests_enabled: false
"""

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.wafv2 import describe_wafv2_tags
from ansible_collections.community.aws.plugins.module_utils.wafv2 import wafv2_list_web_acls


def get_web_acl(wafv2, name, scope, id, fail_json_aws):
    try:
        response = wafv2.get_web_acl(Name=name, Scope=scope, Id=id)
    except (BotoCoreError, ClientError) as e:
        fail_json_aws(e, msg="Failed to get wafv2 web acl.")
    return response


def main():
    arg_spec = dict(
        name=dict(type="str", required=True),
        scope=dict(type="str", required=True, choices=["CLOUDFRONT", "REGIONAL"]),
    )

    module = AnsibleAWSModule(
        argument_spec=arg_spec,
        supports_check_mode=True,
    )

    state = module.params.get("state")
    name = module.params.get("name")
    scope = module.params.get("scope")

    wafv2 = module.client("wafv2")
    # check if web acl exists
    response = wafv2_list_web_acls(wafv2, scope, module.fail_json_aws)

    id = None
    arn = None
    retval = {}

    for item in response.get("WebACLs"):
        if item.get("Name") == name:
            id = item.get("Id")
            arn = item.get("ARN")

    if id:
        existing_acl = get_web_acl(wafv2, name, scope, id, module.fail_json_aws)
        retval = camel_dict_to_snake_dict(existing_acl.get("WebACL"))
        tags = describe_wafv2_tags(wafv2, arn, module.fail_json_aws)
        retval["tags"] = tags

    module.exit_json(**retval)


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type

import pytest

from awx.main.models import WorkflowJobTemplate, NotificationTemplate


@pytest.mark.django_db
def test_create_workflow_job_template(run_module, admin_user, organization, survey_spec):
    result = run_module(
        'workflow_job_template',
        {
            'name': 'foo-workflow',
            'organization': organization.name,
            'extra_vars': {'foo': 'bar', 'another-foo': {'barz': 'bar2'}},
            'survey_spec': survey_spec,
            'survey_enabled': True,
            'state': 'present',
            'job_tags': '',
            'skip_tags': '',
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)

    wfjt = WorkflowJobTemplate.objects.get(name='foo-workflow')
    assert wfjt.extra_vars == '{"foo": "bar", "another-foo": {"barz": "bar2"}}'

    result.pop('invocation', None)
    assert result == {"name": "foo-workflow", "id": wfjt.id, "changed": True}

    assert wfjt.organization_id == organization.id
    assert wfjt.survey_spec == survey_spec


@pytest.mark.django_db
def test_create_modify_no_survey(run_module, admin_user, organization, survey_spec):
    result = run_module(
        'workflow_job_template',
        {
            'name': 'foo-workflow',
            'organization': organization.name,
            'job_tags': '',
            'skip_tags': '',
        },
        admin_user,
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', False), result

    wfjt = WorkflowJobTemplate.objects.get(name='foo-workflow')
    assert wfjt.organization_id == organization.id
    assert wfjt.survey_spec == {}
    result.pop('invocation', None)
    assert result == {"name": "foo-workflow", "id": wfjt.id, "changed": True}

    result = run_module('workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name}, admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert not result.get('changed', True), result


@pytest.mark.django_db
def test_survey_spec_only_changed(run_module, admin_user, organization, survey_spec):
    wfjt = WorkflowJobTemplate.objects.create(organization=organization, name='foo-workflow', survey_enabled=True, survey_spec=survey_spec)
    result = run_module('workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'state': 'present'}, admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert not result.get('changed', True), result
    wfjt.refresh_from_db()
    assert wfjt.survey_spec == survey_spec

    survey_spec['description'] = 'changed description'

    result = run_module(
        'workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'survey_spec': survey_spec, 'state': 'present'}, admin_user
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', True), result
    wfjt.refresh_from_db()
    assert wfjt.survey_spec == survey_spec


@pytest.mark.django_db
def test_survey_spec_missing_field(run_module, admin_user, organization, survey_spec):
    wfjt = WorkflowJobTemplate.objects.create(organization=organization, name='foo-workflow', survey_enabled=True, survey_spec=survey_spec)
    result = run_module('workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'state': 'present'}, admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert not result.get('changed', True), result
    wfjt.refresh_from_db()
    assert wfjt.survey_spec == survey_spec

    del survey_spec['description']

    result = run_module(
        'workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'survey_spec': survey_spec, 'state': 'present'}, admin_user
    )
    assert result.get('failed', True)
    assert result.get('msg') == "Failed to update survey: Field 'description' is missing from survey spec."


@pytest.mark.django_db
def test_associate_only_on_success(run_module, admin_user, organization, project):
    wfjt = WorkflowJobTemplate.objects.create(
        organization=organization,
        name='foo-workflow',
        # survey_enabled=True, survey_spec=survey_spec
    )
    create_kwargs = dict(
        notification_configuration={'url': 'http://www.example.com/hook', 'headers': {'X-Custom-Header': 'value123'}, 'password': 'bar'},
        notification_type='webhook',
        organization=organization,
    )
    nt1 = NotificationTemplate.objects.create(name='nt1', **create_kwargs)
    nt2 = NotificationTemplate.objects.create(name='nt2', **create_kwargs)

    wfjt.notification_templates_error.add(nt1)

    # test preservation of error NTs when success NTs are added
    result = run_module(
        'workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'notification_templates_success': ['nt2']}, admin_user
    )
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', True), result

    assert list(wfjt.notification_templates_success.values_list('id', flat=True)) == [nt2.id]
    assert list(wfjt.notification_templates_error.values_list('id', flat=True)) == [nt1.id]

    # test removal to empty list
    result = run_module('workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'notification_templates_success': []}, admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', True), result

    assert list(wfjt.notification_templates_success.values_list('id', flat=True)) == []
    assert list(wfjt.notification_templates_error.values_list('id', flat=True)) == [nt1.id]


@pytest.mark.django_db
def test_delete_with_spec(run_module, admin_user, organization, survey_spec):
    WorkflowJobTemplate.objects.create(organization=organization, name='foo-workflow', survey_enabled=True, survey_spec=survey_spec)
    result = run_module('workflow_job_template', {'name': 'foo-workflow', 'organization': organization.name, 'state': 'absent'}, admin_user)
    assert not result.get('failed', False), result.get('msg', result)
    assert result.get('changed', True), result

    assert WorkflowJobTemplate.objects.filter(name='foo-workflow', organization=organization).count() == 0



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lambda_alias
version_added: 5.0.0
short_description: Creates, updates or deletes AWS Lambda function aliases
description:
  - This module allows the management of AWS Lambda functions aliases via the Ansible
    framework.  It is idempotent and supports "Check" mode.    Use module M(amazon.aws.lambda) to manage the lambda function
    itself and M(amazon.aws.lambda_event) to manage event source mappings.
  - This module was originally added to C(community.aws) in release 1.0.0.

author:
  - Pierre Jodouin (@pjodouin)
  - Ryan Scott Brown (@ryansb)
options:
  function_name:
    description:
      - The name of the function alias.
    required: true
    type: str
  state:
    description:
      - Describes the desired state.
    default: "present"
    choices: ["present", "absent"]
    type: str
  name:
    description:
      - Name of the function alias.
    required: true
    aliases: ['alias_name']
    type: str
  description:
    description:
      - A short, user-defined function alias description.
    type: str
  function_version:
    description:
      -  Version associated with the Lambda function alias.
         A value of 0 (or omitted parameter) sets the alias to the $LATEST version.
    aliases: ['version']
    type: int
    default: 0
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
---
# Simple example to create a lambda function and publish a version
- hosts: localhost
  gather_facts: false
  vars:
    state: present
    project_folder: /path/to/deployment/package
    deployment_package: lambda.zip
    account: 123456789012
    production_version: 5
  tasks:
    - name: AWS Lambda Function
      amazon.aws.lambda:
        state: "{{ state | default('present') }}"
        name: myLambdaFunction
        publish: true
        description: lambda function description
        code_s3_bucket: package-bucket
        code_s3_key: "lambda/{{ deployment_package }}"
        local_path: "{{ project_folder }}/{{ deployment_package }}"
        runtime: python2.7
        timeout: 5
        handler: lambda.handler
        memory_size: 128
        role: "arn:aws:iam::{{ account }}:role/API2LambdaExecRole"

    - name: Get information
      amazon.aws.lambda_info:
        name: myLambdaFunction
      register: lambda_info
    - name: show results
      ansible.builtin.debug:
        msg: "{{ lambda_info['lambda_facts'] }}"

    # The following will set the Dev alias to the latest version ($LATEST) since version is omitted (or = 0)
    - name: "alias 'Dev' for function {{ lambda_info.lambda_facts.FunctionName }} "
      amazon.aws.lambda_alias:
        state: "{{ state | default('present') }}"
        function_name: "{{ lambda_info.lambda_facts.FunctionName }}"
        name: Dev
        description: Development is $LATEST version

    # The QA alias will only be created when a new version is published (i.e. not = '$LATEST')
    - name: "alias 'QA' for function {{ lambda_info.lambda_facts.FunctionName }} "
      amazon.aws.lambda_alias:
        state: "{{ state | default('present') }}"
        function_name: "{{ lambda_info.lambda_facts.FunctionName }}"
        name: QA
        version: "{{ lambda_info.lambda_facts.Version }}"
        description: "QA is version {{ lambda_info.lambda_facts.Version }}"
      when: lambda_info.lambda_facts.Version != "$LATEST"

    # The Prod alias will have a fixed version based on a variable
    - name: "alias 'Prod' for function {{ lambda_info.lambda_facts.FunctionName }} "
      amazon.aws.lambda_alias:
        state: "{{ state | default('present') }}"
        function_name: "{{ lambda_info.lambda_facts.FunctionName }}"
        name: Prod
        version: "{{ production_version }}"
        description: "Production is version {{ production_version }}"
"""

RETURN = r"""
---
alias_arn:
    description: Full ARN of the function, including the alias
    returned: success
    type: str
    sample: arn:aws:lambda:us-west-2:123456789012:function:myFunction:dev
description:
    description: A short description of the alias
    returned: success
    type: str
    sample: The development stage for my hot new app
function_version:
    description: The qualifier that the alias refers to
    returned: success
    type: str
    sample: $LATEST
name:
    description: The name of the alias assigned
    returned: success
    type: str
    sample: dev
revision_id:
    description: A unique identifier that changes when you update the alias.
    returned: success
    type: str
    sample: 12345678-1234-1234-1234-123456789abc
"""

import re

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.exceptions import AnsibleAWSError
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


class LambdaAnsibleAWSError(AnsibleAWSError):
    pass


def set_api_params(module_params, param_names):
    """
    Sets non-None module parameters to those expected by the boto3 API.

    :param module_params:
    :param param_names:
    :return:
    """

    api_params = dict()

    for param in param_names:
        module_param = module_params.get(param, None)
        if module_param:
            api_params[param] = module_param

    return snake_dict_to_camel_dict(api_params, capitalize_first=True)


def validate_params(module_params):
    """
    Performs basic parameter validation.

    :param module_params: AnsibleAWSModule Parameters
    :return:
    """

    function_name = module_params["function_name"]

    # validate function name
    if not re.search(r"^[\w\-:]+$", function_name):
        raise LambdaAnsibleAWSError(
            f"Function name {function_name} is invalid. Names must contain only alphanumeric characters and hyphens."
        )
    if len(function_name) > 64:
        raise LambdaAnsibleAWSError(f"Function name '{function_name}' exceeds 64 character limit")
    return


def normalize_params(module_params):
    params = dict(module_params)

    #  if parameter 'function_version' is zero, set it to $LATEST, else convert it to a string
    if params["function_version"] == 0:
        params["function_version"] = "$LATEST"
    else:
        params["function_version"] = str(params["function_version"])

    return params


def get_lambda_alias(module_params, client):
    """
    Returns the lambda function alias if it exists.

    :param module_params: AnsibleAWSModule parameters
    :param client: (wrapped) boto3 lambda client
    :return:
    """

    # set API parameters
    api_params = set_api_params(module_params, ("function_name", "name"))

    # check if alias exists and get facts
    try:
        results = client.get_alias(aws_retry=True, **api_params)
    except is_boto3_error_code("ResourceNotFoundException"):
        results = None
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        raise LambdaAnsibleAWSError("Error retrieving function alias", exception=e)

    return results


def lambda_alias(module_params, client, check_mode):
    """
    Adds, updates or deletes lambda function aliases.

    :param module_params: AnsibleAWSModule parameters
    :param client: (wrapped) boto3 lambda client
    :return dict:
    """
    results = dict()
    changed = False
    current_state = "absent"
    state = module_params["state"]

    facts = get_lambda_alias(module_params, client)
    if facts:
        current_state = "present"

    if state == "present":
        if current_state == "present":
            snake_facts = camel_dict_to_snake_dict(facts)

            # check if alias has changed -- only version and description can change
            alias_params = ("function_version", "description")
            for param in alias_params:
                if module_params.get(param) is None:
                    continue
                if module_params.get(param) != snake_facts.get(param):
                    changed = True
                    break

            if changed:
                api_params = set_api_params(module_params, ("function_name", "name"))
                api_params.update(set_api_params(module_params, alias_params))

                if not check_mode:
                    try:
                        results = client.update_alias(aws_retry=True, **api_params)
                    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                        raise LambdaAnsibleAWSError("Error updating function alias", exception=e)

        else:
            # create new function alias
            api_params = set_api_params(module_params, ("function_name", "name", "function_version", "description"))

            try:
                if not check_mode:
                    results = client.create_alias(aws_retry=True, **api_params)
                changed = True
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                raise LambdaAnsibleAWSError("Error creating function alias", exception=e)

    else:  # state = 'absent'
        if current_state == "present":
            # delete the function
            api_params = set_api_params(module_params, ("function_name", "name"))

            try:
                if not check_mode:
                    results = client.delete_alias(aws_retry=True, **api_params)
                changed = True
            except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
                raise LambdaAnsibleAWSError("Error deleting function alias", exception=e)

    return dict(changed=changed, **dict(results or facts or {}))


def main():
    """
    Main entry point.

    :return dict: ansible facts
    """
    argument_spec = dict(
        state=dict(required=False, default="present", choices=["present", "absent"]),
        function_name=dict(required=True),
        name=dict(required=True, aliases=["alias_name"]),
        function_version=dict(type="int", required=False, default=0, aliases=["version"]),
        description=dict(required=False, default=None),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[],
        required_together=[],
    )

    client = module.client("lambda", retry_decorator=AWSRetry.jittered_backoff())

    try:
        validate_params(module.params)
        module_params = normalize_params(module.params)
        results = lambda_alias(module_params, client, module.check_mode)
    except LambdaAnsibleAWSError as e:
        if e.exception:
            module.fail_json_aws(e.exception, msg=e.message)
        module.fail_json(msg=e.message)

    module.exit_json(**camel_dict_to_snake_dict(results))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = r"""
---
module: digital_ocean_sshkey
short_description: Manage DigitalOcean SSH keys
description:
     - Create/delete DigitalOcean SSH keys.
author: "Patrick Marques (@pmarques)"
options:
  state:
    description:
     - Indicate desired state of the target.
    default: present
    choices: ['present', 'absent']
    type: str
  fingerprint:
    description:
     - This is a unique identifier for the SSH key used to delete a key
    aliases: ['id']
    type: str
  name:
    description:
     - The name for the SSH key
    type: str
  ssh_pub_key:
    description:
     - The Public SSH key to add.
    type: str
notes:
  - Version 2 of DigitalOcean API is used.
requirements:
  - "python >= 2.6"
extends_documentation_fragment:
  - community.digitalocean.digital_ocean.documentation
"""


EXAMPLES = r"""
- name: "Create ssh key"
  community.digitalocean.digital_ocean_sshkey:
    oauth_token: "{{ oauth_token }}"
    name: "My SSH Public Key"
    ssh_pub_key: "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAAAQQDDHr/jh2Jy4yALcK4JyWbVkPRaWmhck3IgCoeOO3z1e2dBowLh64QAM+Qb72pxekALga2oi4GvT+TlWNhzPH4V example"
    state: present
  register: result

- name: "Delete ssh key"
  community.digitalocean.digital_ocean_sshkey:
    oauth_token: "{{ oauth_token }}"
    state: "absent"
    fingerprint: "3b:16:bf:e4:8b:00:8b:b8:59:8c:a9:d3:f0:19:45:fa"
"""


RETURN = r"""
# Digital Ocean API info https://docs.digitalocean.com/reference/api/api-reference/#tag/SSH-Keys
data:
    description: This is only present when C(state=present)
    returned: when C(state=present)
    type: dict
    sample: {
        "ssh_key": {
            "id": 512189,
            "fingerprint": "3b:16:bf:e4:8b:00:8b:b8:59:8c:a9:d3:f0:19:45:fa",
            "name": "My SSH Public Key",
            "public_key": "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAAAQQDDHr/jh2Jy4yALcK4JyWbVkPRaWmhck3IgCoeOO3z1e2dBowLh64QAM+Qb72pxekALga2oi4GvT+TlWNhzPH4V example"
        }
    }
"""

import json
import hashlib
import base64

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.digitalocean.plugins.module_utils.digital_ocean import (
    DigitalOceanHelper,
)


class Response(object):
    def __init__(self, resp, info):
        self.body = None
        if resp:
            self.body = resp.read()
        self.info = info

    @property
    def json(self):
        if not self.body:
            if "body" in self.info:
                return json.loads(self.info["body"])
            return None
        try:
            return json.loads(self.body)
        except ValueError:
            return None

    @property
    def status_code(self):
        return self.info["status"]


class Rest(object):
    def __init__(self, module, headers):
        self.module = module
        self.headers = headers
        self.baseurl = module.params.get("baseurl")

    def _url_builder(self, path):
        if path[0] == "/":
            path = path[1:]
        return "%s/%s" % (self.baseurl, path)

    def send(self, method, path, data=None, headers=None):
        url = self._url_builder(path)
        data = self.module.jsonify(data)
        timeout = self.module.params["timeout"]

        resp, info = fetch_url(
            self.module,
            url,
            data=data,
            headers=self.headers,
            method=method,
            timeout=timeout,
        )

        # Exceptions in fetch_url may result in a status -1, the ensures a
        if info["status"] == -1:
            self.module.fail_json(msg=info["msg"])

        return Response(resp, info)

    def get(self, path, data=None, headers=None):
        return self.send("GET", path, data, headers)

    def put(self, path, data=None, headers=None):
        return self.send("PUT", path, data, headers)

    def post(self, path, data=None, headers=None):
        return self.send("POST", path, data, headers)

    def delete(self, path, data=None, headers=None):
        return self.send("DELETE", path, data, headers)


def core(module):
    api_token = module.params["oauth_token"]
    state = module.params["state"]
    fingerprint = module.params["fingerprint"]
    name = module.params["name"]
    ssh_pub_key = module.params["ssh_pub_key"]

    rest = Rest(
        module,
        {
            "Authorization": "Bearer {0}".format(api_token),
            "Content-type": "application/json",
        },
    )

    fingerprint = fingerprint or ssh_key_fingerprint(module, ssh_pub_key)
    response = rest.get("account/keys/{0}".format(fingerprint))
    status_code = response.status_code
    json = response.json

    if status_code not in (200, 404):
        module.fail_json(
            msg="Error getting ssh key [{0}: {1}]".format(
                status_code, response.json["message"]
            ),
            fingerprint=fingerprint,
        )

    if state in ("present"):
        if status_code == 404:
            # IF key not found create it!

            if module.check_mode:
                module.exit_json(changed=True)

            payload = {"name": name, "public_key": ssh_pub_key}
            response = rest.post("account/keys", data=payload)
            status_code = response.status_code
            json = response.json
            if status_code == 201:
                module.exit_json(changed=True, data=json)

            module.fail_json(
                msg="Error creating ssh key [{0}: {1}]".format(
                    status_code, response.json["message"]
                )
            )

        elif status_code == 200:
            # If key found was found, check if name needs to be updated
            if name is None or json["ssh_key"]["name"] == name:
                module.exit_json(changed=False, data=json)

            if module.check_mode:
                module.exit_json(changed=True)

            payload = {
                "name": name,
            }
            response = rest.put("account/keys/{0}".format(fingerprint), data=payload)
            status_code = response.status_code
            json = response.json
            if status_code == 200:
                module.exit_json(changed=True, data=json)

            module.fail_json(
                msg="Error updating ssh key name [{0}: {1}]".format(
                    status_code, response.json["message"]
                ),
                fingerprint=fingerprint,
            )

    elif state in ("absent"):
        if status_code == 404:
            module.exit_json(changed=False)

        if module.check_mode:
            module.exit_json(changed=True)

        response = rest.delete("account/keys/{0}".format(fingerprint))
        status_code = response.status_code
        json = response.json
        if status_code == 204:
            module.exit_json(changed=True)

        module.fail_json(
            msg="Error creating ssh key [{0}: {1}]".format(
                status_code, response.json["message"]
            )
        )


def ssh_key_fingerprint(module, ssh_pub_key):
    try:
        key = ssh_pub_key.split(None, 2)[1]
        fingerprint = hashlib.md5(base64.b64decode(key)).hexdigest()
        return ":".join(a + b for a, b in zip(fingerprint[::2], fingerprint[1::2]))
    except IndexError:
        module.fail_json(
            msg="This does not appear to be a valid public key. Please verify the format and value provided in ssh_public_key."
        )


def main():
    argument_spec = DigitalOceanHelper.digital_ocean_argument_spec()
    argument_spec.update(
        state=dict(choices=["present", "absent"], default="present"),
        fingerprint=dict(aliases=["id"], required=False),
        name=dict(required=False),
        ssh_pub_key=dict(required=False),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=(("fingerprint", "ssh_pub_key"),),
        supports_check_mode=True,
    )

    core(module)


if __name__ == "__main__":
    main()



# (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
# (c) 2015 Toshio Kuratomi <tkuratomi@ansible.com>
# (c) 2017, Peter Sprygada <psprygad@redhat.com>
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


from ansible import constants as C
from ansible.utils.display import Display
from ansible_collections.ansible.utils.plugins.plugin_utils.connection_base import (
    PersistentConnectionBase,
)


display = Display()


__all__ = ["NetworkConnectionBase"]

BUFSIZE = 65536


class NetworkConnectionBase(PersistentConnectionBase):
    """
    A base class for network-style connections using a sub-plugin.
    """

    def __init__(self, play_context, new_stdin, *args, **kwargs):
        super(NetworkConnectionBase, self).__init__(play_context, new_stdin, *args, **kwargs)
        self._network_os = self._play_context.network_os
        self._sub_plugin = {}
        self._cached_variables = (None, None, None)

    def __getattr__(self, name):
        try:
            return self.__dict__[name]
        except KeyError:
            if not name.startswith("_"):
                plugin = self._sub_plugin.get("obj")
                if plugin:
                    method = getattr(plugin, name, None)
                    if method is not None:
                        return method
            raise AttributeError(
                "'%s' object has no attribute '%s'" % (self.__class__.__name__, name)
            )

    def get_options(self, hostvars=None):
        options = super(NetworkConnectionBase, self).get_options(hostvars=hostvars)

        if self._sub_plugin.get("obj") and self._sub_plugin.get("type") != "external":
            try:
                options.update(self._sub_plugin["obj"].get_options(hostvars=hostvars))
            except AttributeError:
                pass

        return options

    def set_options(self, task_keys=None, var_options=None, direct=None):
        super(NetworkConnectionBase, self).set_options(
            task_keys=task_keys, var_options=var_options, direct=direct
        )
        if self.get_option("persistent_log_messages"):
            warning = (
                "Persistent connection logging is enabled for %s. This will log ALL interactions"
                % self._play_context.remote_addr
            )
            logpath = getattr(C, "DEFAULT_LOG_PATH")
            if logpath is not None:
                warning += " to %s" % logpath
            self.queue_message(
                "warning",
                "%s and WILL NOT redact sensitive configuration like passwords. USE WITH CAUTION!"
                % warning,
            )

        if self._sub_plugin.get("obj") and self._sub_plugin.get("type") != "external":
            try:
                self._sub_plugin["obj"].set_options(
                    task_keys=task_keys, var_options=var_options, direct=direct
                )
            except AttributeError:
                pass



# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the nxos_hostname module
"""


class HostnameArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_hostname module"""

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {"type": "dict", "options": {"hostname": {"type": "str"}}},
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "parsed",
                "gathered",
                "rendered",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: compliance_device_status_count_info
short_description: Information module for Compliance Device Status Count
description:
- Get all Compliance Device Status Count.
- Return Compliance Status Count.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  complianceStatus:
    description:
    - >
      ComplianceStatus query parameter. Compliance status can have value among 'COMPLIANT', 'NON_COMPLIANT',
      'IN_PROGRESS', 'NOT_AVAILABLE', 'NOT_APPLICABLE', 'ERROR'.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Compliance GetComplianceStatusCount
  description: Complete reference of the GetComplianceStatusCount API.
  link: https://developer.cisco.com/docs/dna-center/#!get-compliance-status-count
notes:
  - SDK Method used are
    compliance.Compliance.get_compliance_status_count,

  - Paths used are
    get /dna/intent/api/v1/compliance/count,

"""

EXAMPLES = r"""
- name: Get all Compliance Device Status Count
  cisco.dnac.compliance_device_status_count_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    complianceStatus: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": 0
    }
"""
